## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## Run detection of drifting signals in signal series (compression- and shear wave)
##
## Usage: [r_ofp] = anaers_run_algo(p_tp, p_ap)
##
## p_tp  ... material type enumerator, <str>
##             'cem1_25': cement pastes, CEM I 42.5 N (test series 1), distance = 25mm
##             'cem1_50': cement pastes, CEM I 42.5 N (test series 1), distance = 50mm
##             'cem1_70': cement pastes, CEM I 42.5 N (test series 1), distance = 70mm
## p_ap  ... analysis parameter data structure, <struct>
## r_ofp ... return: analysis result output file path, <str>
##
## see also: anadd_dsdefs_cem1, anadd_dsdefs_air, anadd_dsdefs_water
##
function [r_ofp] = anaers_run_algo(p_tp, p_ap)
  
  ## check arguments
  if (nargin < 1)
    p_tp = 'cem1_50';
  endif
  if (nargin < 2)
    ## read analysis parameters
    ap = anaers_param();
  else
    ap = p_ap;
  endif
  
  ## read dataset definitions
  switch (p_tp)
    case 'cem1_25'
      ## distance = 25mm, 5 w/c-ratios, 3 turns => 15 datasets
      defs = anaers_dsdefs_cem1();
      r_ofp = hlp_run_group(ap, defs, defs.def.d25.p2, p_tp);
    case 'cem1_50'
      ## distance = 50mm, 5 w/c-ratios, 3 turns => 15 datasets
      defs = anaers_dsdefs_cem1();
      r_ofp = hlp_run_group(ap, defs, defs.def.d50.p2, p_tp);
    case 'cem1_70'
      ## distance = 70mm, 5 w/c-ratios, 3 turns => 15 datasets
      defs = anaers_dsdefs_cem1();
      r_ofp = hlp_run_group(ap, defs, defs.def.d70.p2, p_tp);
  endswitch
  
endfunction

########################################################################################################################
function [r_ofp] = hlp_run_group(p_ap, p_defs, p_gdef, p_dstp)
  ## Helper function: run analysis for one dataset group
  ## p_ap   ... analysis parameter data structure, <struct>
  ## p_defs ... dataset definition structure, <struct>
  ## p_gdef ... dataset group definition, <struct>
  ## p_dstp ... dataset type, one of {'cem1_25', 'cem1_50', 'cem1_70'}, <str>
  ## r_ofp  ... return: output file path, <str>
  
  ## select dataset group from dataset list
  def = p_defs.dsl(p_gdef.idx);
  ndef = max(size(def));
  
  ## print status
  printf('anaers: start analysis for dataset group %s\n', p_gdef.code);
  
  ## init batch data structure
  ds0.obj = 'anaers_struct_analysis_batch'; # object name
  ds0.ver = int16([1, 0]); # object version
  ds0.defs = p_defs; # dataset definition data structure
  ds0.gdef = p_gdef; # dataset group selection data structure
  ds0.def = def; # dataset definition list of selected group
  ds0.serid = [];
  ds0.sercode = [];
  ds0.sername = [];
  ds0.setid = [];
  ds0.setcode = [];
  ds0.setname = [];
  ds0.dstype = struct_objattrib('dstype', p_dstp, 'dataset type');
  ds0.chnum = struct_objdata('chnum', 'uint', 0, '', 'channel number');
  ds0.chname = struct_objdata('chname', 'str', '', '', 'channel name');
  ds0.dsfn = struct_objattrib('dsfn', '', 'dataset file name');
  ds0.dsfp = struct_objattrib('dsfp', '', 'dataset file path, full qualified');
  ds0.btm = []; # analysis time benchmark
  ds0.par0 = []; # initial analysis parameter data structure
  ds0.par1 = []; # modified analysis parameter data structure
  ds0.st01 = []; # PTS statistics, all signals
  ds0.st02 = []; # ERS statistics, all signals
  ds0.st11 = []; # PTS statistics, all signals, after bias removal
  ds0.st12 = []; # ERS statistics, all signals, after bias removal
  ds0.st21 = []; # PTS statistics, selected signals, after outlyer detection
  ds0.st22 = []; # ERS statistics, selected signals, after outlyer detection
  ds0.detol = []; # outlyer detection data structure
  ds0.ananoi = []; # noise removal data structure
  ds0.anasig = []; # min/max array data structure
  ds0.anawla = []; # wavelet analysis data structure
  ## channel 1
  ds1 = ds0;
  ds1.chnum.v = 1;
  ds1.chname.v = 'C-wave';
  ## channel 2
  ds2 = ds0;
  ds2.chnum.v = 2;
  ds2.chname.v = 'S-wave';
  
  ## iterate over dataset group definitions
  for i = 1 : ndef
    ## dataset file path
    dspath = fullfile(p_ap.dspath, p_defs.dsdir, def(i).fn);
    dsfno = struct_objattrib('dsfn', def(i).fn, 'dataset file name');
    dsfpo = struct_objattrib('dsfp', dspath, 'dataset file path, full qualified');
    ## load dataset
    ds = fio_struct_load(dspath);
    ## update batch data structure, test series metadata
    if (i == 1)
      metaser = struct_exp_data(ds, 'meta_ser');
      ## channel 1, c-wave
      ds1.serid = metaser.d01;
      ds1.sercode = metaser.a01;
      ds1.sername = metaser.a02;
      ## channel 2, s-wave
      ds2.serid = metaser.d01;
      ds2.sercode = metaser.a01;
      ds2.sername = metaser.a02;
    endif
    ## update batch data structure, dataset metadata
    metaset = struct_exp_data(ds, 'meta_set');
    ## channel 1, c-wave
    ds1.setid = [ds1.setid; metaset.d01];
    ds1.setcode = [ds1.setcode; metaset.a01];
    ds1.setname = [ds1.setname; metaset.a02];
    ## channel 2, s-wave
    ds2.setid = [ds2.setid; metaset.d01];
    ds2.setcode = [ds2.setcode; metaset.a01];
    ds2.setname = [ds2.setname; metaset.a02];
    ## analyse channel 1, c-wave
    cn = 1;
    anac1 = anaers_algo(p_ap, ds, cn, p_dstp);
    ## analyse channel 2, s-wave
    cn = 2;
    anac2 = anaers_algo(p_ap, ds, cn, p_dstp);
    ## update batch data structure, analysis results, channel 1, c-wave
    ds1.dsfn = [ds1.dsfn; dsfno];
    ds1.dsfp = [ds1.dsfp; dsfpo];
    ds1.btm = [ds1.btm; anac1.btm];
    ds1.par0 = [ds1.par0; anac1.par0];
    ds1.par1 = [ds1.par1; anac1.par1];
    ds1.st01 = [ds1.st01; anac1.st01];
    ds1.st02 = [ds1.st02; anac1.st02];
    ds1.st11 = [ds1.st11; anac1.st11];
    ds1.st12 = [ds1.st12; anac1.st12];
    ds1.st21 = [ds1.st21; anac1.st21];
    ds1.st22 = [ds1.st22; anac1.st22];
    ds1.detol = [ds1.detol; anac1.detol];
    ds1.ananoi = [ds1.ananoi; anac1.ananoi];
    ds1.anasig = [ds1.anasig; anac1.anasig];
    ds1.anawla = [ds1.anawla; anac1.anawla];
    ## update batch data structure, analysis results, channel 2, s-wave
    ds2.dsfn = [ds2.dsfn; dsfno];
    ds2.dsfp = [ds2.dsfp; dsfpo];
    ds2.btm = [ds2.btm; anac2.btm];
    ds2.par0 = [ds2.par0; anac2.par0];
    ds2.par1 = [ds2.par1; anac2.par1];
    ds2.st01 = [ds2.st01; anac2.st01];
    ds2.st02 = [ds2.st02; anac2.st02];
    ds2.st11 = [ds2.st11; anac2.st11];
    ds2.st12 = [ds2.st12; anac2.st12];
    ds2.st21 = [ds2.st21; anac2.st21];
    ds2.st22 = [ds2.st22; anac2.st22];
    ds2.detol = [ds2.detol; anac2.detol];
    ds2.ananoi = [ds2.ananoi; anac2.ananoi];
    ds2.anasig = [ds2.anasig; anac2.anasig];
    ds2.anawla = [ds2.anawla; anac2.anawla];
  endfor
  
  ## compute group statistics
  ## wavelet shift, wavelet start index
  p1 = [[ds1.anawla].bmws];
  p2 = [[ds2.anawla].bmws];
  pc = 'i_w';
  pn = 'Wavelet shift index';
  gstats.bmws = hlp_grp_stats(p1, p2, pc, pn);
  ## wavelet length
  p1 = [[ds1.anawla].bmwl];
  p2 = [[ds2.anawla].bmwl];
  pc = 'l_w';
  pn = 'Wavelet length';
  gstats.bmwl = hlp_grp_stats(p1, p2, pc, pn);
  
  ## print status
  printf('anaers: finished analysis for dataset group %s\n', p_gdef.code);
  
  ## post processing
  ## save analysis results to binary file (GNU octave native binary format)
  if (p_ap.post.save_bin)
    fn = sprintf('%s.oct', p_gdef.code);
    r_ofp = fullfile(p_ap.scpath, p_defs.ardir, fn);
    save('-binary', r_ofp, 'ds1', 'ds2', 'gstats');
    printf('anaers: Saved binary file: %s\n', r_ofp);
  else
    r_ofp = '';
  endif
  
endfunction

function [r_ds] = hlp_grp_stats(p_p1, p_p2, p_pc, p_pn)
  ## Create test group statistics data structure
  ## p_p1 ... channel 1 (c-wave) property object array, [<ADE/struct>]
  ## p_p2 ... channel 2 (s-wave) property object array, [<ADE/struct>]
  ## p_pc ... property code, short name, <str>
  ## p_pn ... property name, full name, <str>
  
  v1 = [p_p1.v];
  v2 = [p_p2.v];
  v0 = [v1, v2];
  u = p_p1(1).u;
  vt = p_p1(1).vt;
  pc1 = sprintf('%s^{(C)}', p_pc);
  pc2 = sprintf('%s^{(S)}', p_pc);
  pc0 = sprintf('%s^{(C, S)}', p_pc);
  pn1 = sprintf('%s (C)', p_pn);
  pn2 = sprintf('%s (S)', p_pn);
  pn0 = sprintf('%s (C, S)', p_pn);
  vn1 = {'C,wc040,t4', 'C,wc040,t5', 'C,wc040,t6', ...
  'C,wc045,t4', 'C,wc045,t5', 'C,wc045,t6', ...
  'C,wc050,t4', 'C,wc050,t5', 'C,wc050,t6', ...
  'C,wc055,t4', 'C,wc055,t5', 'C,wc055,t6', ...
  'C,wc060,t4', 'C,wc060,t5', 'C,wc060,t6'};
  vn2 = {'S,wc040,t4', 'S,wc040,t5', 'S,wc040,t6', ...
  'S,wc045,t4', 'S,wc045,t5', 'S,wc045,t6', ...
  'S,wc050,t4', 'S,wc050,t5', 'S,wc050,t6', ...
  'S,wc055,t4', 'S,wc055,t5', 'S,wc055,t6', ...
  'S,wc060,t4', 'S,wc060,t5', 'S,wc060,t6'};
  vn0 = [vn1, vn2];
  
  r_ds.obj = 'anaers_struct_grp_stats'; # object name
  r_ds.ver = int16([1, 0]); # object version
  ## channel 1, c-wave
  r_ds.statcode1 = struct_objattrib('statcode1', pc1, 'property code, channel 1');
  r_ds.statname1 = struct_objattrib('statname1', p_pn, 'property name, channel 1');
  r_ds.statval1 = struct_objdata('statval1', sprintf('%s_arr', vt), v1, u, 'property value array, channel 1');
  r_ds.statlbl1 = struct_objdata('statlbl1', 'str_arr', vn1, '', 'property label array, channel 1');
  r_ds.statmin1 = struct_objdata('statmin1', vt, min(v1), u, 'minimum of property value array, channel 1');
  r_ds.statmax1 = struct_objdata('statmax1', vt, max(v1), u, 'maximum of property value array, channel 1');
  r_ds.statmean1 = struct_objdata('statmean1', 'double', mean(v1), u, 'mean of property value array, channel 1');
  ## channel 2, s-wave
  r_ds.statcode2 = struct_objattrib('statcode2', pc2, 'property code, channel 2');
  r_ds.statname2 = struct_objattrib('statname2', p_pn, 'property name, channel 2');
  r_ds.statval2 = struct_objdata('statval2', sprintf('%s_arr', vt), v2, u, 'property value array, channel 2');
  r_ds.statlbl2 = struct_objdata('statlbl2', 'str_arr', vn2, '', 'property label array, channel 2');
  r_ds.statmin2 = struct_objdata('statmin2', vt, min(v2), u, 'minimum of property value array, channel 2');
  r_ds.statmax2 = struct_objdata('statmax2', vt, max(v2), u, 'maximum of property value array, channel 2');
  r_ds.statmean2 = struct_objdata('statmean2', 'double', mean(v2), u, 'mean of property value array, channel 2');
  ## both channels
  r_ds.statcode0 = struct_objattrib('statcode0', pc0, 'property code, both channels');
  r_ds.statname0 = struct_objattrib('statname0', p_pn, 'property name, both channels');
  r_ds.statval0 = struct_objdata('statval0', sprintf('%s_arr', vt), v0, u, 'property value array, both channels');
  r_ds.statlbl0 = struct_objdata('statlbl0', 'str_arr', vn0, '', 'property label array, both channels');
  r_ds.statmin0 = struct_objdata('statmin0', vt, min(v0), u, 'minimum of property value array, both channels');
  r_ds.statmax0 = struct_objdata('statmax0', vt, max(v0), u, 'maximum of property value array, both channels');
  r_ds.statmean0 = struct_objdata('statmean0', 'double', mean(v0), u, 'mean of property value array, both channels');
  
  ## group w/c-ratios together
  pmat = [];
  plbl = {};
  i = 1;
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v1,   1:3, 'C,wc040,*');
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v1,   4:6, 'C,wc045,*');
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v1,   7:9, 'C,wc050,*');
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v1, 10:12, 'C,wc055,*');
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v1, 13:15, 'C,wc060,*');
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v2,   1:3, 'S,wc040,*');
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v2,   4:6, 'S,wc045,*');
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v2,   7:9, 'S,wc050,*');
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v2, 10:12, 'S,wc055,*');
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v2, 13:15, 'S,wc060,*');
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v1,  1:15, 'C,*,*');
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v2,  1:15, 'S,*,*');
  [pmat, plbl] = hlp_add_grp_stats(pmat, plbl, v0,  1:30, '*,*,*');
  r_ds.statplttab = struct_objdata('statplttab', 'dbl_mat', pmat, '', 'stats for plotting');
  r_ds.statpltlbl = struct_objdata('statpltlbl', 'str_arr', plbl, '', 'labels for plotting');
  
endfunction

function [r_pmat, r_plbl] = hlp_add_grp_stats(p_pmat, p_plbl, p_val, p_idx, p_lbl)
  ## Add row to test group statistics, statistics of subset of entire test group
  ## p_pmat ... property statistics matrix to be updated, {{<dbl>}}
  ## p_plbl ... property label array, {<str>}
  ## p_val  ... property value array (entire test group), <uint>
  ## p_idx  ... test group index range, subset index, [<uint>]
  ## p_lbl  ... test group label, subset label, <str>
  ## r_pmat ... return: updated property matrix, {{<dbl>}}
  ## r_plbl ... return: updated property label array, {<str>}
  
  vmin = min(p_val(p_idx));
  vmax = max(p_val(p_idx));
  vmean = mean(p_val(p_idx));
  vmed = median(p_val(p_idx));
  vd1 = vmax - vmean;
  vd2 = vmean - vmin;
  vd3 = vmax - vmed;
  vd4 = vmed - vmin;
  r = [vmin, vmax, vmean, vd1, vd2, vmed, vd3, vd4];
  r_pmat = [p_pmat; r];
  r_plbl = [p_plbl; p_lbl];
  
endfunction
