############################################################################################################################################
## wavupv - signal analysis for wave propagation problems (ultrasonic pulse velocity method)
## Copyright (C) 2022  Jakob Harden (Graz University of Technology), jakob.harden@tugraz.at
## This file is part of wavupv and part of the PhD thesis of Jakob Harden (Graz University of Technology)
############################################################################################################################################
##
## Run detection of drifting signals in signal series (compression- and shear wave)
##
## Usage: [r_ofp] = anaers_run_stats(p_tp, p_ap)
##
## p_tp  ... material type enumerator, <str>
##             'cem1_25': cement pastes, CEM I 42.5 N (test series 1), distance = 25mm
##             'cem1_50': cement pastes, CEM I 42.5 N (test series 1), distance = 50mm
##             'cem1_70': cement pastes, CEM I 42.5 N (test series 1), distance = 70mm
## p_ap  ... analysis parameter data structure, <struct>
## r_ofp ... analysis result output file path, <str>
##
## see also: anadd_dsdefs_cem1, anadd_dsdefs_air, anadd_dsdefs_water
##
function [r_ofp] = anaers_run_export(p_tp, p_ap)
  
  ## check arguments
  if (nargin < 1)
    p_tp = 'cem1_50';
  endif
  if (nargin < 2)
    ## read analysis parameters
    ap = anaers_param();
  else
    ap = p_ap;
  endif
  
  ## read dataset definitions
  switch (p_tp)
    case 'cem1_25'
      ## distance = 25mm, 5 wc-ratios, 3 turns => 15 datasets
      defs = anaers_dsdefs_cem1();
      gdef = defs.def.d25.p2;
    case 'cem1_50'
      ## distance = 50mm, 5 wc-ratios, 3 turns => 15 datasets
      defs = anaers_dsdefs_cem1();
      gdef = defs.def.d50.p2;
    case 'cem1_70'
      ## distance = 70mm, 5 wc-ratios, 3 turns => 15 datasets
      defs = anaers_dsdefs_cem1();
      gdef = defs.def.d70.p2;
  endswitch
  
  ## export signal data
  if (ap.post.save_tex)
    hlp_export_data(ap, defs, gdef);
  endif
  
endfunction

function hlp_export_data(p_ap, p_defs, p_gdef)
  ## Helper function: export signal data for plotting
  ## p_ap   ... analysis parameter data structure, <struct>
  ## p_defs ... dataset definition structure, <struct>
  ## p_gdef ... dataset group definition, <struct>
  
  ## select dataset group from dataset list
  def = p_defs.dsl(p_gdef.idx);
  ndef = max(size(def));
  
  ## print status
  printf('anaers: export signal data to TeX code, dataset group %s\n', p_gdef.code);
  
  ## read analysis result data structure
  ifn = sprintf('%s.oct', p_gdef.code);
  ifp = fullfile(p_ap.scpath, p_defs.ardir, ifn);
  ds1 = load(ifp).ds1;
  ds2 = load(ifp).ds2;
  gstats = load(ifp).gstats;
  
  ## output directory
  odir = fullfile(p_ap.scpath, p_defs.exdir);
  
  ## export min/max array results
  hlp_export_items(ds1.anasig, odir, sprintf('%s.%s.%s', p_gdef.code, 'C', 'anasig'));
  hlp_export_items(ds2.anasig, odir, sprintf('%s.%s.%s', p_gdef.code, 'S', 'anasig'));
  
  ## export wavelet analysis results
  hlp_export_items(ds1.anawla, odir, sprintf('%s.%s.%s', p_gdef.code, 'C', 'anawla'));
  hlp_export_items(ds2.anawla, odir, sprintf('%s.%s.%s', p_gdef.code, 'S', 'anawla'));
  
  ## export group statistics
  hlp_export_items(gstats.bmws, odir, sprintf('%s.%s', p_gdef.code, 'gstats.bmws'));
  hlp_export_items(gstats.bmwl, odir, sprintf('%s.%s', p_gdef.code, 'gstats.bmwl'));
  
endfunction

function hlp_export_items(p_sds, p_od, p_px, p_si)
  ## Helper function: export signal data for plotting
  ## p_sds ... sub data structure, <struct>
  ## p_od  ... output directory, <str>
  ## p_px  ... filename/variable name prefix, <str>
  ## p_si  ... selected item index, optional, [<uint>]
  
  ## check arguments
  if (nargin < 4)
    p_si = 1 : max(size(p_sds));
  endif
  
  ## export items
  if (numel(p_sds) == 1)
    hlp_export_fields(p_sds, p_od, p_px);
  else
    for i = p_si
      px = sprintf('%s.%d', p_px, i);
      hlp_export_fields(p_sds(i), p_od, px);
    endfor
  endif
  
endfunction

function hlp_export_fields(p_ds, p_od, p_px)
  ## Helper function: run statistics on analysis results
  ## p_ds ... data structure, <struct>
  ## p_px ... filename/variable name prefix, <str>
  ## p_dn ... data structure name, <str>
  
  for i = 1 : max(size(p_ds))
    ofp = fullfile(p_od, p_px);
    r_ec = tex_struct_export(p_ds, p_px, ofp);
  endfor
  
endfunction
