## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## Run test series statistics on analysis results (cement paste tests)
##
## Usage: [r_ofp] = anaers_run_tsstats()
##
## r_ofp ... return: output file path, <str>
##
## see also: anaers_run_algo
##
function [r_ofp] = anaers_run_tsstats()
  
  ## read analysis parameters
  ap = anaers_param();
  
  ## expected/estimated wavelet length
  par_b = ap.wla.par_b;
  par_d = ap.wla.par_d;
  par_c = fix(anaers_est_discharge());
  ewl = par_b + par_c * 2 + par_d;
  
  ## read analysis results
  fp1 = fullfile(ap.scpath, 'results', 'cem1', 'ts1_d25_p2.oct');
  fp2 = fullfile(ap.scpath, 'results', 'cem1', 'ts1_d50_p2.oct');
  fp3 = fullfile(ap.scpath, 'results', 'cem1', 'ts1_d70_p2.oct');
  ar1 = load(fp1);
  ar2 = load(fp2);
  ar3 = load(fp3);
  
  ## compute min/max/median of all tests
  ws25c = ar1.gstats.bmws.statval1.v;
  ws25s = ar1.gstats.bmws.statval2.v;
  ws50c = ar2.gstats.bmws.statval1.v;
  ws50s = ar2.gstats.bmws.statval2.v;
  ws70c = ar3.gstats.bmws.statval1.v;
  ws70s = ar3.gstats.bmws.statval2.v;
  ws25 = [ws25c, ws25s];
  ws50 = [ws50c, ws50s];
  ws70 = [ws70c, ws70s];
  wsc = [ws25c, ws50c, ws70c];
  wss = [ws25s, ws50s, ws70s];
  ws = [ws25, ws50, ws70];
  wl25c = ar1.gstats.bmwl.statval1.v;
  wl25s = ar1.gstats.bmwl.statval2.v;
  wl50c = ar2.gstats.bmwl.statval1.v;
  wl50s = ar2.gstats.bmwl.statval2.v;
  wl70c = ar3.gstats.bmwl.statval1.v;
  wl70s = ar3.gstats.bmwl.statval2.v;
  wl25 = [wl25c, wl25s];
  wl50 = [wl50c, wl50s];
  wl70 = [wl70c, wl70s];
  wlc = [wl25c, wl50c, wl70c];
  wls = [wl25s, wl50s, wl70s];
  wl = [wl25, wl50, wl70];
  
  ## collect rows of statistics table
  tab = {};
  tab = hlp_addtabrow(tab, '25', 'C', median(ws25c), median(wl25c), ewl);
  tab = hlp_addtabrow(tab, '25', 'S', median(ws25s), median(wl25s), ewl);
  tab = hlp_addtabrow(tab, '25', '*', median(ws25), median(wl25), ewl);
  tab = hlp_addtabrow(tab, '50', 'C', median(ws50c), median(wl50c), ewl);
  tab = hlp_addtabrow(tab, '50', 'S', median(ws50s), median(wl50s), ewl);
  tab = hlp_addtabrow(tab, '50', '*', median(ws50), median(wl50), ewl);
  tab = hlp_addtabrow(tab, '70', 'C', median(ws70c), median(wl70c), ewl);
  tab = hlp_addtabrow(tab, '70', 'S', median(ws70s), median(wl70s), ewl);
  tab = hlp_addtabrow(tab, '70', '*', median(ws70), median(wl70), ewl);
  tab = hlp_addtabrow(tab, '*', 'C', median(wsc), median(wlc), ewl);
  tab = hlp_addtabrow(tab, '*', 'S', median(wss), median(wls), ewl);
  tab = hlp_addtabrow(tab, '*', '*', median(ws), median(wl), ewl);
  hdr1 = {'$D$', '$W$', '$s_w$', '$l_w$', '$\Delta l_w$', '$N_{tot}$', '$t_{tot}$'};
  hdr2 = {'$mm$', '$\#$', '$\#$', '$\#$', '$\#$', '$\#$', '$\mu sec$'};
  
  ## create export data structure
  ds.obj = 'struct_anaers_tsstats';
  ds.ver = int16([1, 0]);
  ds.statstab = struct_objdata('statstab', 'str_mat', tab, '', 'statistics table');
  ds.hdrnames = struct_objdata('hdrnames', 'str_arr', hdr1, '', 'statistics table');
  ds.hdrunits = struct_objdata('hdrunits', 'str_arr', hdr2, '', 'statistics table');

  ## export statistics to TeX files
  if (ap.post.save_tex)
    r_ofp = fullfile(ap.scpath, 'export', 'cem1', 'tsstats');
    errcode = tex_struct_export(ds, 'tsstats', r_ofp);
  endif
  
endfunction

function [r_tab] = hlp_addtabrow(p_tab, p_di, p_wt, p_ws, p_wl, p_ewl)
  ## Add row to statistics table
  ## p_tab ... statistics table, {{<str>}}
  ## p_di  ... measurement distance, <str>
  ## p_wt  ... wave type (C = c-wave, S = s-wave), <str>
  ## p_ws  ... wavelet shift, <uint>
  ## p_wl  ... wavelet length, <uint>
  ## p_ewl ... expected wavelet length, disturbance range, <uint>
  ## r_tab ... return: updated statistics table, {{<str>}}
  
  dlw = p_wl - p_ewl;
  ntot = p_ws + p_wl;
  ttot = ntot / 10;
  trow = {p_di, p_wt, sprintf('%d', p_ws), sprintf('%d', p_wl), sprintf('%d', dlw), sprintf('%d', ntot), sprintf('%.1f', ttot)};
  r_tab = [p_tab; trow];
  
endfunction
