## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## Run electromegantic response section (ERS) analysis on a series of signals (one channel)
##
## Usage: [r_ds] = anaers_algo(p_ap, p_ds, p_cn)
##
## p_ap ... analysis parameter data structure, <struct>
## p_ds ... dataset data structure, <struct>
## p_cn ... channel number, <uint>
## p_dt ... dataset type, <str>
## r_ds ... return: analysis result data structure, <struct>
##
## see also: anaers_run_algo, anaers_wavelet_xcorr2
##
function [r_ds] = anaers_algo(p_ap, p_ds, p_cn, p_dt)
  
  ## init return data structure
  r_ds = [];
  r_ds.btm = struct_objdata('tmbmk', 'double_arr', [], 'sec', 'analysis time benchmark');
  r_ds.par0 = p_ap; # initial analysis parameter data structure
  r_ds.par1 = []; # modified analysis parameter data structure
  r_ds.gdef = []; # dataset group definition
  r_ds.st01 = []; # PTS statistics, all signals
  r_ds.st02 = []; # ERS statistics, all signals
  r_ds.st11 = []; # PTS statistics, all signals, after bias removal
  r_ds.st12 = []; # ERS statistics, all signals, after bias removal
  r_ds.st21 = []; # PTS statistics, selected signals, after outlyer detection
  r_ds.st22 = []; # ERS statistics, selected signals, after outlyer detection
  r_ds.detol = []; # outlyer detection data structure
  r_ds.ananoi = []; # noise removal data structure
  r_ds.anasig = []; # min/max array data structure
  r_ds.anawla = []; # wavelet analysis data structure
  
  #######################################
  ## start time benchmark
  r_ds.btm.v = hlp_benchmark_time();
  
  ## dataset code
  dco = struct_exp_data(p_ds, 'meta_set_code');
  
  ## signal data
  switch (p_cn)
    case 1
      i0o = struct_exp_data(p_ds, 'tst.utt1.i0');
      tso = struct_exp_data(p_ds, 'tst.utt1.ts');
      mso = struct_exp_data(p_ds, 'tst.utt1.ms');
      pwo = struct_exp_data(p_ds, 'tst.utt1.pw');
      sro = struct_exp_data(p_ds, 'tst.utt1.sr');
      t0o = struct_exp_data(p_ds, 'tst.utt1.t0');
      mao = struct_exp_data(p_ds, 'tst.utt1.mat');
    case 2
      i0o = struct_exp_data(p_ds, 'tst.utt2.i0');
      tso = struct_exp_data(p_ds, 'tst.utt2.ts');
      mso = struct_exp_data(p_ds, 'tst.utt2.ms');
      pwo = struct_exp_data(p_ds, 'tst.utt2.pw');
      sro = struct_exp_data(p_ds, 'tst.utt2.sr');
      t0o = struct_exp_data(p_ds, 'tst.utt2.t0');
      mao = struct_exp_data(p_ds, 'tst.utt2.mat');
  endswitch
  
  ## add zero time to maturity array
  mao.v = mao.v + t0o.v;
  
  ## update analysis parameter wavelet start/end length
  ## use pulse width for rough estimate of wavelet length (disturbance range)
  ## ln0, ln1 = 2 * c +/- 10, see anaers_wavelet.m for details
  ## where c = pulse_width * sampling_rate
  c2 = fix(pwo.v * sro.v) * 2;
  ## update analysis parameters, wavelet length range
  p_ap.wla.ln0 = c2 - 10; # lower limit of variable part of wavelet length (2 * c)
  p_ap.wla.ln1 = c2 + 10; # upper limit of variable part of wavelet length (2 * c)
  
  ## maturity/signal index range, select all signals
  irmats = 1 : size(mso.v, 2);
  
  ## index range of pre-trigger section (PTS section)
  irpts0 = 1;
  irpts1 = i0o.v - 1;
  irpts = transpose([irpts0 : irpts1]);
  
  ## index range of electromagnetic response section (ERS section)
  ## setup index range
  irers0 = i0o.v;
  ## switch dataset type
  switch (p_dt)
    case 'cem1_25'
      irers1 = i0o.v + p_ap.nar25 - 1;
    case 'cem1_50'
      irers1 = i0o.v + p_ap.nar50 - 1;
    case 'cem1_70'
      irers1 = i0o.v + p_ap.nar70 - 1;
    otherwise
      irers1 = i0o.v + p_ap.narfb - 1;
  endswitch
  irers = [irers0 : irers1];
  printf('anaers_algo: using analysis range (sample index range): %d to %d\n', irers0, irers1);
  
  ## pre-trigger section statistics (PTS stats), all signals
  r_ds.st01 = hlp_stats(mso, tso, mao, irpts, irmats, dco.v, p_cn);
  
  ## electromagnetic response section statistics (ERS stats), all signals
  r_ds.st02 = hlp_stats(mso, tso, mao, irers, irmats, dco.v, p_cn);
  
  #######################################
  ## remove bias
  ## bias is the mean of the pre-trigger section
  if p_ap.mod.rmbias
    mso.v = mso.v - r_ds.st01.statmean.v;
  endif
  
  ## pre-trigger section statistics (PTS stats), all signals, after bias removal
  r_ds.st11 = hlp_stats(mso, tso, mao, irpts, irmats, dco.v, p_cn);
  
  ## electromagnetic response section statistics (ERS stats), all signals, after bias removal
  r_ds.st12 = hlp_stats(mso, tso, mao, irers, irmats, dco.v, p_cn);
  
  #######################################
  ## find signals with outlyers
  if p_ap.mod.outlyers
    r_ds.detol = hlp_det_outlyers(mso.v, irmats, irers0, irers1, p_ap.thol, dco.v, p_cn);
    ## update selected signal index array
    irmats = r_ds.detol.sigidxad.v;
  else
    r_ds.detol = hlp_det_outlyers();
  endif
  
  ## pre-trigger section statistics (PTS stats), all signals, after outlyer detection
  r_ds.st21 = hlp_stats(mso, tso, mao, irpts, irmats, dco.v, p_cn);
  
  ## electromagnetic response section statistics (ERS stats), all signals, after outlyer detection
  r_ds.st22 = hlp_stats(mso, tso, mao, irers, irmats, dco.v, p_cn);
  
  ## print number of recognized signals (after removing outlyers by threshold detection)
  cstat_tot = size(mso.v, 2);
  cstat_use = max(size(irmats));
  cstat_out = cstat_tot - cstat_use;
  cstat_prc = cstat_use / cstat_tot * 100;
  printf('anaers_algo: total: %d, used: %d, skiped: %d, percentage of used signals: %.1f\n', cstat_tot, cstat_use, cstat_out, cstat_prc);
  
  #######################################
  ## run wavelet analysis
  [r_ds.ananoi, r_ds.anasig, r_ds.anawla] = hlp_wavelet(mso, tso, mao, irmats, irers0, irers1, p_ap, dco.v, p_cn);
  ## if wavelet length variation is at lower limit, shift wavelet length range by minus 10
  if (r_ds.anawla.errl.v == 1)
    ## update wavelet length range (shrink by 10 samples)
    p_ap.wla.ln0 = p_ap.wla.ln0 - 10;
    p_ap.wla.ln1 = p_ap.wla.ln1 - 10;
    [r_ds.ananoi, r_ds.anasig, r_ds.anawla] = hlp_wavelet(mso, tso, mao, irmats, irers0, irers1, p_ap, dco.v, p_cn);
    disp('anaers_algo: second turn necessary, because wavelet length is at LOWER limit');
  endif
  ## if wavelet length variation is at upper limit, shift wavelet length range by plus 10
  if (r_ds.anawla.errl.v == 2)
    ## update wavelet length range (extend by 10 samples)
    p_ap.wla.ln0 = p_ap.wla.ln0 + 10;
    p_ap.wla.ln1 = p_ap.wla.ln1 + 10;
    [r_ds.ananoi, r_ds.anasig, r_ds.anawla] = hlp_wavelet(mso, tso, mao, irmats, irers0, irers1, p_ap, dco.v, p_cn);
    disp('anaers_algo: second turn necessary, because wavelet length is at UPPER limit');
  endif
  
  #######################################
  ## save modified analysis parameter structure
  r_ds.par1 = p_ap;
  
  #######################################
  ## stop time benchmark
  r_ds.btm.v = hlp_benchmark_time(r_ds.btm.v);
  
endfunction

function [r_vt] = hlp_benchmark_time(p_vt)
  ## Benchmark time tool
  ##
  ## Usage 1: [r_vt] = benchmark_time() ... start computation time benchmark
  ## Usage 2: [r_vt] = benchmark_time(p_vt) ... end computation time benchmark (provide r_vt from benchmark start as argument p_vt)
  ##
  ## p_vt ... time vector [1x3], optional, [<dbl>]
  ## r_vt ... return: time vector, if p_vt is omitted return [now(), 0, 0], otherwise return [t0, now(), delta_t], delta_t in seconds, [<dbl>]
  ##
  ## Example:
  ## start benchmark: initial_vt = benchmark_time()
  ## ... run computations
  ## end benchmark: updated_vt = benchmark_time(initial_vt)
  
  ## check arguments
  if (nargin < 1)
    r_vt = [now(), 0, 0];
    return;
  endif
  
  ## update
  if (max(size(p_vt)) == 3)
    r_vt = p_vt;
    r_vt(2) = now();
    r_vt(3) = (r_vt(2) - r_vt(1)) * 3600 * 24;
  else
    r_vt = p_vt;
  endif
  
endfunction

function [r_ds] = hlp_meta(p_sc, p_cn)
  ## Create dataset metadata structure
  ## p_sc  ... dataset code, <str>
  ## p_cn  ... channel number, <uint>
  ## r_ds  ... return: dataset group definition data structure, <struct>
  
  r_ds.obj = [];
  r_ds.ver = [];
  r_ds.algoname = [];
  r_ds.algoversion = [];
  r_ds.chnum = struct_objattrib('chnum', sprintf('%d', p_cn), 'channel number');
  if (p_cn == 1)
    r_ds.chncode = struct_objattrib('chncode', 'C', 'channel code');
    r_ds.chnamelong = struct_objattrib('chnamelong', 'Compression wave', 'channel name, long');
    r_ds.chnameshort = struct_objattrib('chnameshort', 'C-wave', 'channel name, short');
  else
    r_ds.chncode = struct_objattrib('chncode', 'S', 'channel code');
    r_ds.chnamelong = struct_objattrib('chnamelong', 'Shear wave', 'channel name, long');
    r_ds.chnameshort = struct_objattrib('chnameshort', 'S-wave', 'channel name, short');
  endif
  r_ds.dscode = struct_objattrib('dscode', p_sc, 'dataset code');
  r_ds.dscodetex = struct_objattrib('dscodetex', strrep(p_sc, '_', '\_'), 'dataset code, TeX escaped');

endfunction

function [r_ds] = hlp_stats(p_mso, p_tso, p_mao, p_sx, p_mx, p_sc, p_cn)
  ## Compute statistics
  ## p_mso ... signal magnitude matrix, volts, [nsmp x nsig], <ADE/struct>
  ## p_tso ... signal sample time signature array, sec, [nsmp x 1], <ADE/struct>
  ## p_mao ... signal maturity array, sec, [nsig x 1], <ADE/struct>
  ## p_sx  ... signal sample index array, selected sample range, [uint]
  ## p_mx  ... specimen maturity index array, selected signals, [uint]
  ## p_sc  ... dataset code, <str>
  ## p_cn  ... channel number, <uint>
  ## r_ds  ... return: statistics data structure, <struct>
  
  ## init data structure with metadata
  r_ds = hlp_meta(p_sc, p_cn);
  
  ## prepare signal data
  ms = p_mso.v(p_sx, p_mx);
  msunit = p_mso.u;
  ts = p_tso.v(p_sx, 1);
  ma = p_mao.v(p_mx, 1);
  
  ## update data structure, add fields
  r_ds.obj = 'struct_ers_stats';
  r_ds.ver = int16([1, 0]);
  r_ds.algoname = 'ERS signal statistics';
  r_ds.algoversion = int16([1, 0]);
  r_ds.sigts = struct_objdata('sigts', 'double_arr', ts, 'sec', 'sample time series, column vector');
  r_ds.sigmat = struct_objdata('spcmat', 'double_arr', ma, 'sec', 'specimen maturity array, column vector');
  r_ds.sigidx = struct_objdata('sigidx', 'uint', p_mx, '#', 'maturity index array');
  r_ds.spmidx = struct_objdata('smpidx', 'uint', p_sx, '#', 'sample index range array');
  r_ds.spmidx0 = struct_objdata('smpidx0', 'uint', p_sx(1), '#', 'sample index range, start index');
  r_ds.spmidx1 = struct_objdata('smpidx1', 'uint', p_sx(end), '#', 'sample index range, end index');
  r_ds.statmax = struct_objdata('statmax', 'double_arr', max(ms), msunit, 'maximum voltage, row vector');
  r_ds.statmin = struct_objdata('statmin', 'double_arr', min(ms), msunit, 'minimum voltage, row vector');
  r_ds.statrng = struct_objdata('statrng', 'double_arr', range(ms), msunit, 'range of voltage, row vector');
  r_ds.statmean = struct_objdata('statmean', 'double_arr', mean(ms), msunit, 'mean voltage, row vector');
  r_ds.statrms = struct_objdata('statrms', 'double_arr', rms(ms), msunit, 'root-mean square of voltage, row vector');
  r_ds.statstd = struct_objdata('statstd', 'double_arr', std(ms), msunit, 'standard deviation voltage, row vector');
  r_ds.statmaxabs = struct_objdata('statmaxabs', 'double_arr', max(abs(ms)), msunit, 'maximum of absolute voltage, row vector');

endfunction

function [r_ds] = hlp_det_outlyers(p_ms, p_mx, p_i0, p_i1, p_th, p_sc, p_cn)
  ## Detect outlyers by threshold detection based on quantiles
  ## p_ms ... signal magnitude matrix, [[<double>]]
  ## p_mx ... signal maturity index of selected signals, [<uint>]
  ## p_i0 ... sample index range, start index, <uint>
  ## p_i1 ... sample index range, end index, <uint>
  ## p_th ... detection threshold value, number between [0, 1], <double>
  ## p_sc ... dataset code, <str>
  ## p_cn ... channel number, <uint>
  ## r_ds ... return: analysis data structure, <struct>
  
  if (nargin == 7)
    ## detect outlyers
    ## detection status
    detstat = true;
    ## detection sample index range
    irng = [p_i0 : p_i1];
    ## detection vector
    v = max(abs(p_ms(irng, p_mx)));
    ## find percentile defined by threshold
    plim = [0, p_th * 100, 100];
    pval = prctile(v, plim);
    ixloc = find(v > pval(2));
    ix = p_mx(ixloc);
    if not(isempty(ix))
      mx = setdiff(p_mx, ix);
    endif
  else
    ## no outlyer detection
    ## detection status
    detstat = false;
    ## provide default values
    p_th = 0;
    p_i0 = 0;
    p_i1 = 0;
    p_sc = '';
    p_cn = 0;
    irng = [];
    p_mx = [];
    plim = [];
    pval = [];
    v = [];
    mx = [];
    ix = [];
  endif
  
  ## init data structure with metadata
  r_ds = hlp_meta(p_sc, p_cn);
  
  ## create analysis data structure
  r_ds.obj = 'struct_det_outlyers';
  r_ds.ver = int16([1, 0]);
  r_ds.algoname = 'ERS outlyer detection';
  r_ds.algover = int16([1, 0]);
  r_ds.detstat = struct_objdata('detstat', 'bool', detstat, '/', 'detection status, true = detect, false = skip detection');
  r_ds.detthv = struct_objdata('detthv', 'double', p_th, '/', 'detection threshold value, quantile, value out of [0,1]');
  r_ds.detvec = struct_objdata('detvec', 'double_arr', v, 'V', 'detection vector, max(abs(x)) of selected signals');
  r_ds.detplim = struct_objdata('detplim', 'double_arr', plim, '%', 'detection percentile limits, based on given threshold value');
  r_ds.detpval = struct_objdata('detpval', 'double_arr', pval, 'V', 'detection percentile values, based on given threshold value');
  r_ds.sidx = struct_objdata('smpidx', 'uint_arr', irng, '#', 'detection sample index range');
  r_ds.sidx0 = struct_objdata('smpidx0', 'uint', p_i0, '#', 'sample index range, start index');
  r_ds.sidx1 = struct_objdata('smpidx1', 'uint', p_i1, '#', 'sample index range, end index');
  r_ds.sigidxbd = struct_objdata('sigidxbd', 'uint_arr', p_mx, '#', 'signal index, before detection');
  r_ds.sigidxad = struct_objdata('sigidxad', 'uint_arr', mx, '#', 'signal index, after detection');
  r_ds.sigidxol = struct_objdata('sigidxol', 'uint_arr', ix, '#', 'signal index, outlyers');
  r_ds.nsigbd = struct_objdata('nsigbd', 'uint', max(size(p_mx)), '#', 'number of signals before detection');
  r_ds.nsigad = struct_objdata('nsigad', 'uint', max(size(mx)), '#', 'number of signals after detection');
  r_ds.nsigol = struct_objdata('nsigol', 'uint', max(size(ix)), '#', 'number of outlyers');

endfunction

function [r_dsnoi, r_dssig, r_dswla] = hlp_wavelet(p_mso, p_tso, p_mao, p_mx, p_i0, p_i1, p_ap, p_sc, p_cn)
  ## Detect phase shift and disturbance time range of electromagnetic response using the wavelet analysis
  ## p_mso   ... signal magnitude matrix, Volts, [[<double>]]
  ## p_tso   ... signal time signature array, sec, [<uint>]
  ## p_mao   ... signal maturity array, sec, [<uint>]
  ## p_mx    ... signal maturity index of selected signals, [<uint>]
  ## p_i0    ... analysis index range, start index, <uint>
  ## p_i1    ... analysis index range, end index, <uint>
  ## p_ap    ... analysis parameter structure, <struct>
  ## p_sc    ... dataset code, <str>
  ## p_cn    ... channel number, <uint>
  ## r_dsnoi ... return: noise removal data structure, <struct>
  ## r_dssig ... return: min/max array data structure, <struct>
  ## r_dswla ... return: wavelet analysis data structure, <struct>
  
  ########################################
  ## measurement units
  u_ms = p_mso.u; # measurement unit, signal magnitudes
  u_ts = p_tso.u; # measurement unit, sample time signature
  u_ma = p_mao.u; # measurement unit, maturity time signature
  
  ########################################
  ## ERS, select signal data
  ms_ers = p_mso.v(p_i0 : p_i1, p_mx);
  ts_ers = p_tso.v(p_i0 : p_i1, 1);
  ma_ers = p_mao.v(p_mx);
  
  ########################################
  ## PTS, select signal data
  si0_pts = 1;
  si1_pts = p_i0 - 1;
  ms_pts = p_mso.v(si0_pts : si1_pts, p_mx);
  ts_pts = p_tso.v(si0_pts : si1_pts, 1);
  ma_pts = p_mao.v(p_mx);
  
  ########################################
  ## noise threshold
  thv_pts = max(max(ms_pts));
  ## noise filter matrix
  fltmat_pts = abs(ms_ers) >= thv_pts;
  ## remove noise
  if p_ap.mod.rmnoise
    ms_ers = ms_ers .* fltmat_pts;
  endif
  
  ## noise data structure
  r_dsnoi = hlp_meta(p_sc, p_cn);
  r_dsnoi.obj = 'struct_rmnoise';
  r_dsnoi.ver = int16([1, 0]);
  r_dsnoi.algoname = 'ERS noise removal';
  r_dsnoi.algover = int16([1, 0]);
  r_dsnoi.rmnoistat = struct_objdata('rmnoistat', 'bool', p_ap.mod.rmnoise, '', 'noise removal status, true = removed noise, false = no action');
  r_dsnoi.sigidx = struct_objdata('sigidx', 'uint_arr', p_mx, '', 'index array of selected signals, PTS section');
  r_dsnoi.sigmat = struct_objdata('sigmat', 'uint_arr', ma_pts, u_ma, 'maturity array of selected signals, PTS section');
  r_dsnoi.smpidx = struct_objdata('smpid0', 'uint_arr', [si0_pts : si1_pts], '', 'sample index range, PTS section');
  r_dsnoi.smpidx0 = struct_objdata('smpidx0', 'uint', si0_pts, '', 'sample index start index, PTS section');
  r_dsnoi.smpidx1 = struct_objdata('smpidx1', 'uint', si1_pts, '', 'sample index end index, PTS section');
  r_dsnoi.smpts = struct_objdata('smpts', 'double_arr', ts_pts, u_ts, 'sample time signature array of selected signals, PTS section');
  r_dsnoi.thv = struct_objdata('thv', 'double', thv_pts, u_ms, 'noise threshold value, max of noise megnitude matrix');
  r_dsnoi.flt = struct_objdata('flt', 'double_mat', fltmat_pts, '', 'noise filter matrix, rows = sample index, columns = signal index');
  
  ########################################
  ## min/max array of filtered signal magnitude matrix
  ## column 1: max curve
  ## column 2: min curve
  c1 = max(ms_ers, [], 2);
  c2 = min(ms_ers, [], 2);
  ers_max = max(c1);
  ers_min = min(c2);
  ers_rng = ers_max - ers_min;
  ers_maxabs = max([abs(ers_max), abs(ers_min)]);
  
  ## create min/max array data structure
  r_dssig = hlp_meta(p_sc, p_cn);
  r_dssig.obj = 'struct_minmax_array';
  r_dssig.ver = int16([1, 0]);
  r_dssig.algoname = 'ERS min/max array';
  r_dssig.algover = int16([1, 0]);
  r_dssig.sigidx = struct_objdata('sigidx', 'uint_arr', p_mx, '', 'index array of selected signals, ERS section');
  r_dssig.sigmat = struct_objdata('sigmat', 'uint_arr', ma_ers, u_ma, 'maturity array of selected signals, ERS section');
  r_dssig.smpidx = struct_objdata('smpidx', 'uint_arr', [p_i0 : p_i1], '', 'sample index range, ERS section');
  r_dssig.smpidx0 = struct_objdata('smpidx0', 'uint', p_i0, '', 'sample index start index, ERS section');
  r_dssig.smpidx1 = struct_objdata('smpidx1', 'uint', p_i1, '', 'sample index end index, ERS section');
  r_dssig.smpts = struct_objdata('smpts', 'double_arr', ts_ers, u_ts, 'sample time signature array of selected signals, ERS section');
  r_dssig.arrmax = struct_objdata('arrmax', 'double_arr', c1, u_ms, 'magnitude maximum curve of selected signals, ERS section, column vector');
  r_dssig.arrmin = struct_objdata('arrmin', 'double_arr', c2, u_ms, 'magnitude minimum curve of selected signals, ERS section, column vector');
  r_dssig.magmax = struct_objdata('magmax', 'double', ers_max, u_ms, 'magnitude maximum of selected signals, ERS section');
  r_dssig.magmin = struct_objdata('magmin', 'double', ers_min, u_ms, 'magnitude minimum of selected signals, ERS section');
  r_dssig.magrng = struct_objdata('magrng', 'double', ers_rng, u_ms, 'magnitude range of selected signals, ERS section');
  r_dssig.magmaxabs = struct_objdata('magmaxabs', 'double', ers_maxabs, u_ms, 'absolute magnitude maximum of selected signals, ERS section');
  
  ## compute cross correlation, wavelet analysis
  r_dswla = anaers_wavelet_xcorr2(p_ap, [c1, c2], p_sc, p_cn);
  
endfunction
