## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## Estimate time range of eletromagnetic response section from device properties
##
## Usage: [r_ns, r_tm] = anaers_est_discharge()
##
## r_ns ... number of samples while discharging, <uint>
## r_tm ... discharging time, seconds, <dbl>
##
## see also: 
##
##  Network:
##
##      cappgen          impsen
##    [capacity]-------[actuator]-------[ground]
##              lcable1          lcable2
##              rcable           rcable
##
function [r_ns, r_tm] = anaers_est_discharge()
  
  ## electromechanical parameters of network
  lcable1 = 1.2; # m, BNC cable length, pulse generator capacity -> actuator
  lcable2 = 1.2; # m, BNC cable length, actuator -> ground of pulse generator
  rcable = 50.0; # Ohm/m, resistance of BNC cable
  impsen = 50.0; # Ohm, impedance of sensor
  cappgen = 2.5; # microFarad, capacity of puls generator
  smprate = 10000000; # Hertz, sampling rate
  
  ## resistance of network
  rnet = (lcable1 + lcable2) * rcable + impsen;
  
  ## estimation for time-constant tau
  tau = rnet * cappgen * 1e-9;
  
  ## estimation for dicharge period, 5 * tau
  tau5 = 5 * tau;
  
  ## estimate for number of samples while discharging
  r_ns = tau5 * smprate;
  
  ## estimate for discharging time (5 time-constants)
  r_tm = tau5;
  
endfunction
