## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## Create 3D-plots of electromagentic response section (disturbance caused by pulse excitation)
##
## Usage: anaers_plot3d()
##
## see also: 
##
function anaers_plot3d()
  
  ## analysis parameter
  ap = anaers_param();
  
  ## define dataset
  ts = 'ts1_cem1_paste'; # test series directory name
  dn = 'ts1_wc050_d50_5'; # dataset code, file name
  
  ## dataset file name and path
  ifn = sprintf('%s.oct', dn); # input, dataset file name
  ifp = fullfile(ap.dspath, ts, 'oct', ifn); # input, dataset file path
  
  ## output file name and path
  ofn = dn; # output file name
  ofp = fullfile(ap.scpath, 'export', 'plot3d', ofn); # output file path without extension
  
  ## load dataset
  ds = fio_struct_load(ifp);
  
  ## sample index range to be plotted
  ## plot first 100 samples beginning with trigger point
  ix = 1000 : 1 : 1100;
  
  ## signal index range to be plotted
  iy = 1 : 1 : 288;
  
  ## plot c-wave, channel 1
  fh1 = hlp_plot_channel(ds, ix, iy, 1);
  
  ## plot s-wave, channel 2
  fh2 = hlp_plot_channel(ds, ix, iy, 2);
  
  ## save plots
  fp1_fig = sprintf('%s_c.fig', ofp);
  fp1_png = sprintf('%s_c.png', ofp);
  fp2_fig = sprintf('%s_s.fig', ofp);
  fp2_png = sprintf('%s_s.png', ofp);
  hgsave(fh1, fp1_fig);
  print(fh1, fp1_png);
  savefig(fh2, fp2_fig);
  print(fh2, fp2_png);

endfunction

function [r_fh] = hlp_plot_channel(p_ds, p_ix, p_iy, p_cn)
  ## Plot channel (c-wave or s-wave)
  ## p_ds ... dataset, <struct>
  ## p_ix ... sample index range, [<uint>]
  ## p_iy ... signal index range, [<uint>]
  ## p_cn ... channel number (1 = c-wave, 2 = s-wave)
  
  ## signal data, plot properties
  switch (p_cn)
    case 1
      ## signal data
      x1 = p_ds.tst.s06.d11.v(p_iy) / 60; # maturity, min
      y1 = p_ds.tst.s06.d12.v(p_ix) * 1e6; # time signature, usec
      z1 = p_ds.tst.s06.d13.v(p_ix, p_iy); # signal magnitude
      ## plot properties
      colmap = 'autumn';
      colmin = 'red';
      colmax = 'red';
      colrng = [255, 176, 0] / 255;
      ## labels
      chnlbl = 'C';
      chntit = 'Compression wave';
    case 2
      ## signal data
      x1 = p_ds.tst.s07.d11.v(p_iy) / 60; # maturity, min
      y1 = p_ds.tst.s07.d12.v(p_ix) * 1e6; # time signature, usec
      z1 = p_ds.tst.s07.d13.v(p_ix, p_iy); # signal magnitude
      ## plot properties
      colmap = 'winter';
      colmin = 'blue';
      colmax = 'blue';
      colrng = [0, 176, 179] / 255;
      ## labels
      chnlbl = 'S';
      chntit = 'Shear wave';
  endswitch
  pltlw = 2;
  fontsz = 12;
  fontnm = 'Times New Roman';
  
  ## prepare data
  [xx1, yy1] = meshgrid(x1, y1);
  rng1 = range(z1);
  min1 = min(z1, [], 2);
  max1 = max(z1, [], 2);
  
  ## plot data
  r_fh = figure('position',[200,200,1024,768], "paperunits", "centimeters", "paperposition", [0, 0, 15, 10]);
  set(gca, "linewidth", 2, "fontsize", fontsz, "fontname", fontnm);
  hold on;
  colormap(colmap);
  grid;
  light("Position", [0.5 -1 1], "Style", 'infinite');
  lighting('gouraud');
  view(3);
  surf(xx1, yy1, z1, 'linestyle', 'none', 'facecolor', 'interp', 'ambientstrength', 0.7);
  plot3(x1, ones(size(x1))*y1(end), rng1, 'linewidth', pltlw, 'color', colrng);
  plot3(ones(size(y1))*2000, y1, min1, 'linewidth', pltlw, 'color', colmin);
  plot3(ones(size(y1))*2000, y1, max1, 'linewidth', pltlw, 'color', colmax);
  xlabel('maturity [min]');
  ylabel('time [\musec]');
  zlabel('magnitude [V]');
  title(chntit);
  hold off;
  
endfunction
