## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## Export signal data, example
##
## Usage: anaers_signal()
##
## see also: 
##
function anaers_signal()
  
  ap = anaers_param();
  
  ## export cem1 example, signal index 200
  dd = fullfile(ap.dspath, 'ts1_cem1_paste');
  hlp_export_ers(ap, dd, 'ts1_wc040_d50_5', 200);
  hlp_export_sig(ap, dd, 'ts1_wc040_d50_5', 200);
  
  ## export cem1 example, signal index 100
  dd = fullfile(ap.dspath, 'ts1_cem1_paste');
  hlp_export_ers(ap, dd, 'ts1_wc040_d50_5', 100);
  hlp_export_sig(ap, dd, 'ts1_wc040_d50_5', 100);
  
  ## export cem1 example, signal index 50
  hlp_export_ers(ap, dd, 'ts1_wc040_d50_5', 50);
  hlp_export_sig(ap, dd, 'ts1_wc040_d50_5', 50);
  
  ## export cem1 example, signal index 25
  hlp_export_ers(ap, dd, 'ts1_wc040_d50_5', 25);
  hlp_export_sig(ap, dd, 'ts1_wc040_d50_5', 25);
  
endfunction

function hlp_export_ers(p_ap, p_dd, p_dn, p_si)
  ## Export electromagnetic response section of signal
  ## p_ap ... analysis parameter structure, <struct>
  ## p_dd ... export data directory, <str>
  ## p_dn ... dataset name, file name, <str>
  ## p_si ... signal index, <uint>
  
  ## dataset file path
  dsfp = fullfile(p_dd, 'oct', sprintf('%s.oct', p_dn));
  
  ## load data structure
  ds = fio_struct_load(dsfp);
  
  ## get signal data
  ms1 = struct_exp_data(ds, 'tst.utt1.ms');
  ms2 = struct_exp_data(ds, 'tst.utt2.ms');
  ts1 = struct_exp_data(ds, 'tst.utt1.ts');
  ma1 = struct_exp_data(ds, 'tst.utt1.mat');
  i01 = struct_exp_data(ds, 'tst.utt1.i0');
  t01 = struct_exp_data(ds, 'tst.utt1.t0');
  
  ## prepare signal data
  di0 = 12;
  di1 = di0 + 60;
  ix = transpose(i01.v : (i01.v + p_ap.narfb - 1));
  m1 = ms1.v(ix, p_si);
  m2 = ms2.v(ix, p_si);
  ts = ts1.v(ix);
  ma = ma1.v(p_si) + t01.v;
  e1min = min(ms1.v(ix, :), [], 2);
  e1max = max(ms1.v(ix, :), [], 2);
  e2min = min(ms2.v(ix, :), [], 2);
  e2max = max(ms2.v(ix, :), [], 2);
  
  ## plot signal data
  ymin = min([min(m1), min(m2)]);
  ymax = max([max(m1), max(m2)]);
  figure();
  hold on;
  plot(ts, m1, sprintf('-r;c_{%d};', p_si)); # signal data, c-wave
  plot(ts, m2, sprintf('-b;s_{%d};', p_si)); # signal data, s-wave
  plot([ts(di0), ts(di0)], [ymin, ymax], '-k'); # ERS, lower limit
  plot([ts(di1), ts(di1)], [ymin, ymax], '-k'); # ERS, upper limit
  xlabel('Sample time [sec]');
  ylabel('Magnitude [V]');
  title(sprintf('ERS, %s', strrep(p_dn, '_', '\_')));
  hold off;
  
  ## signal data matrix
  dat = [ix, ts, m1, m2, e1min, e1max, e2min, e2max];
  
  ## create export data structure
  ## metadata
  ds.dscode = struct_objattrib('dscode', strrep(p_dn, '_', '\_'), 'dataset code');
  ds.ch1code = struct_objattrib('ch1code', 'c', 'channel 1 name, c-wave');
  ds.ch2code = struct_objattrib('ch2code', 's', 'channel 2 name, s-wave');
  ## data
  ds.data = struct_objdata('sig', 'double_mat', dat, '', 'signal data table');
  ds.matt = struct_objdata('matt', 'uint', fix(ma / 60), 'min', 'specimen maturity, time');
  ds.matx = struct_objdata('matx', 'uint', p_si, '', 'specimen maturity, signal index');
  ## signal extrema
  ds.emax = struct_objdata('emax', 'double', max([max(m1), max(m2)]), 'V', 'c,s-wave maximum');
  ds.emax1 = struct_objdata('emax1', 'double', max(m1), 'V', 'c-wave maximum');
  ds.emax2 = struct_objdata('emax2', 'double', max(m2), 'V', 's-wave maximum');
  ds.emin = struct_objdata('emin', 'double', min([min(m1), min(m2)]), 'V', 'c,s-wave minimum');
  ds.emin1 = struct_objdata('emin1', 'double', min(m1), 'V', 'c-wave minimum');
  ds.emin2 = struct_objdata('emin2', 'double', min(m2), 'V', 's-wave minimum');
  ## min/max curve extrema
  ds.mmcmax = struct_objdata('mmcmax', 'double', max([max(e1max), max(e2max)]), 'V', 'min/max curve, c,s-wave maximum');
  ds.mmcmax1 = struct_objdata('mmcmax1', 'double', max(e1max), 'V', 'min/max curve, c-wave maximum');
  ds.mmcmax2 = struct_objdata('mmcmax2', 'double', max(e2max), 'V', 'min/max curve, s-wave maximum');
  ds.mmcmin = struct_objdata('mmcmin', 'double', min([min(e1min), min(e2min)]), 'V', 'min/max curve, c,s-wave minimum');
  ds.mmcmin1 = struct_objdata('mmcmin1', 'double', min(e1min), 'V', 'min/max curve, c-wave minimum');
  ds.mmcmin2 = struct_objdata('mmcmin2', 'double', min(e2min), 'V', 'min/max curve, s-wave minimum');
  ## annotation
  ds.xlbl1 = struct_objattrib('xlbl1', 'Sample index i', 'x axis label');
  ds.xunit1 = struct_objattrib('xunit1', '', 'x axis unit');
  ds.xlbl2 = struct_objattrib('xlbl2', 'Time', 'x axis label');
  ds.xunit2 = struct_objattrib('xunit2', ts1.u, 'x axis unit');
  ds.ylbl = struct_objattrib('ylbl', 'Magnitude', 'y axis label');
  ds.yunit = struct_objattrib('yunit', ms1.u, 'y axis unit');
  ds.title = struct_objattrib('title', sprintf('Signals'), 'figure title');
  ## legend
  ds.leg1 = struct_objattrib('leg1', sprintf('c_{%d}', p_si), 'legend entry, c-wave');
  ds.leg2 = struct_objattrib('leg2', sprintf('s_{%d}', p_si), 'legend entry, s-wave');
  ## marker points
  ds.mp1t = struct_objdata('mp1t', 'uint', ts(1), 'sec', 'marker 1, trigger point, sample time signature');
  ds.mp1x = struct_objdata('mp1x', 'uint', ix(1), '', 'marker 1, trigger point, sample index');
  ds.mp2t = struct_objdata('mp2t', 'uint', ts(end), 'sec', 'marker 2, selection end, sample time signature');
  ds.mp2x = struct_objdata('mp2x', 'uint', ix(end), '', 'marker 2, selection end, sample index');
  ds.mp3t = struct_objdata('mp3t', 'uint', ts(di0), 'sec', 'marker 3, estimated ers section start, sample time signature');
  ds.mp3x = struct_objdata('mp3x', 'uint', ix(di0), '', 'marker 3, estimated ers section start, sample index');
  ds.mp4t = struct_objdata('mp4t', 'uint', ts(di1), 'sec', 'marker 4, estimated ers section end, sample time signature');
  ds.mp4x = struct_objdata('mp4x', 'uint', ix(di1), '', 'marker 4, estimated ers section end, sample index');
  
  ## TeX export
  if (p_ap.post.save_tex)
    vname = sprintf('%s_ers', p_dn);
    ofp = fullfile(p_ap.scpath, 'export', 'signals', sprintf('%s_%d', vname, p_si));
    errcode = tex_struct_export(ds, sprintf('s%d.%s', p_si, vname), ofp);
  endif
  
endfunction

function hlp_export_sig(p_ap, p_dd, p_dn, p_si)
  ## Export complete signal
  ## p_ap ... analysis parameter structure, <struct>
  ## p_dd ... export data directory, <str>
  ## p_dn ... dataset name, file name, <str>
  ## p_si ... signal index, <uint>
  
  ## dataset file path
  dsfp = fullfile(p_dd, 'oct', sprintf('%s.oct', p_dn));
  
  ## load data structure
  ds = fio_struct_load(dsfp);
  
  ## get signal data
  ms1 = struct_exp_data(ds, 'tst.utt1.ms');
  ms2 = struct_exp_data(ds, 'tst.utt2.ms');
  ts1 = struct_exp_data(ds, 'tst.utt1.ts');
  ma1 = struct_exp_data(ds, 'tst.utt1.mat');
  i01 = struct_exp_data(ds, 'tst.utt1.i0');
  t01 = struct_exp_data(ds, 'tst.utt1.t0');
  
  ## prepare signal data
  di0 = i01.v + 12;
  di1 = di0 + 60;
  ix = transpose(1 : 5000);
  m1 = ms1.v(ix, p_si);
  m2 = ms2.v(ix, p_si);
  ts = ts1.v(ix);
  ma = ma1.v(p_si) + t01.v;
  
  ## plot signal data
  ymin = min([min(m1), min(m2)]);
  ymax = max([max(m1), max(m2)]);
  figure();
  hold on;
  plot(ts, m1, sprintf('-r;c_{%d};', p_si)); # signal data, c-wave
  plot(ts, m2, sprintf('-b;s_{%d};', p_si)); # signal data, s-wave
  plot([ts(di0), ts(di0)], [ymin, ymax], '-k'); # ERS, lower limit
  plot([ts(di1), ts(di1)], [ymin, ymax], '-k'); # ERS, upper limit
  xlabel('Sample time [sec]');
  ylabel('Magnitude [V]');
  title(sprintf('Signal, %s', strrep(p_dn, '_', '\_')));
  hold off;
  
  ## signal data matrix
  dat = [ix, ts, m1, m2];
  
  ## create export data structure
  ## metadata
  ds.dscode = struct_objattrib('dscode', strrep(p_dn, '_', '\_'), 'dataset code');
  ds.ch1code = struct_objattrib('ch1code', 'c', 'channel 1 name, c-wave');
  ds.ch2code = struct_objattrib('ch2code', 's', 'channel 2 name, s-wave');
  ## data
  ds.data = struct_objdata('sig', 'double_mat', dat, '', 'signal data table');
  ds.matt = struct_objdata('matt', 'uint', fix(ma / 60), 'min', 'specimen maturity, time');
  ds.matx = struct_objdata('matx', 'uint', p_si, '', 'specimen maturity, signal index');
  ## axis labelst, title
  ds.xlbl1 = struct_objattrib('xlbl1', 'Sample index i', 'x axis label');
  ds.xunit1 = struct_objattrib('xunit1', '', 'x axis unit');
  ds.xlbl2 = struct_objattrib('xlbl2', 'Time', 'x axis label');
  ds.xunit2 = struct_objattrib('xunit2', ts1.u, 'x axis unit');
  ds.ylbl = struct_objattrib('ylbl', 'Magnitude', 'y axis label');
  ds.yunit = struct_objattrib('yunit', ms1.u, 'y axis unit');
  ds.title = struct_objattrib('title', sprintf('Signals'), 'figure title');
  ## legend
  ds.leg1 = struct_objattrib('leg1', sprintf('c_{%d}', p_si), 'legend entry, c-wave');
  ds.leg2 = struct_objattrib('leg2', sprintf('s_{%d}', p_si), 'legend entry, s-wave');
  ## marker points
  ds.mp1t = struct_objdata('mp1t', 'uint', ts(1), 'sec', 'marker 1, signal start, sample time signature');
  ds.mp1x = struct_objdata('mp1x', 'uint', ix(1), '', 'marker 1, signal start, sample index');
  ds.mp2t = struct_objdata('mp2t', 'uint', ts(i01.v), 'sec', 'marker 2, trigger point, sample time signature');
  ds.mp2x = struct_objdata('mp2x', 'uint', i01.v, '', 'marker 2, trigger point, sample index');
  ds.mp3t = struct_objdata('mp3t', 'uint', ts(end), 'sec', 'marker 3, signal end, sample time signature');
  ds.mp3x = struct_objdata('mp3x', 'uint', ix(end), '', 'marker 3, signal end, sample index');
  ds.mp4t = struct_objdata('mp4t', 'uint', ts(di1), 'sec', 'marker 4, estimated ers section end, sample time signature');
  ds.mp4x = struct_objdata('mp4x', 'uint', ix(di1), '', 'marker 4, estimated ers section end, sample index');
  ## signal extrema
  ds.emax = struct_objdata('emax', 'double', max([max(m1), max(m2)]), 'V', 'c,s-wave maximum');
  ds.emin = struct_objdata('emin', 'double', min([min(m1), min(m2)]), 'V', 'c,s-wave minimum');
  
  ## TeX export
  if (p_ap.post.save_tex)
    vname = sprintf('%s_sig', p_dn);
    ofp = fullfile(p_ap.scpath, 'export', 'signals', sprintf('%s_%d', vname, p_si));
    errcode = tex_struct_export(ds, sprintf('s%d.%s', p_si, vname), ofp);
  endif
  
endfunction
