## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## Generate wavelet function for electromagnetic response section detection
##
## Usage: [r_wm, r_wl, r_wc, r_mx, r_mv] = anaers_wavelet(p_ap, p_c2)
##
## p_ap ... analysis parameter structure, <struct>
## p_c2 ... variable part of wavelet length (= 2 * c), <uint>
## r_wm ... return: wavelet function magnitudes [1 x wl], [<float>]
## r_wl ... return: wavelet length (number of samples), <uint>
## r_wc ... return: center of distribution, <uint>
## r_mx ... return: marker point indices, characteristic points [1 x 5], <uint>
## r_mv ... return: marker point magnitudes, characteristic points [1 x 5], <double>
##
## see also: wl_mkstruct
##
##            f=1.0
##            |\_        g=0.7
##            |  \__     |\_
##       e=0.3|     \___ |  \___
##       +--  |         \|      \___
##       |  --|                     \______
##  +----+----------------------------------------+----->
##  |<a >|<b >|<   c    >|<   c    >|<     d     >|
##       |<             wl (DR)                  >|
##       |mp1 |mp2       |mp3       |mp4          |mp5
##       |<                 p1                   >|
##            |<   p2   >|<   p3   >|
##       
##
## a = wavelet shift, variable distance
## b = fix
## c = variable part of wavelet length
## d = fix
## e = fix
## f = fix
## g = fix
## wl = b + 2 * c + d, wavelet length = length of disturbance range
## mp1 to mp5 = marker points used for plots
## p1 to p3 = parts 1 to 3 of the superimposition
##
function [r_wm, r_wl, r_wc, r_mx, r_mv] = anaers_wavelet(p_ap, p_c2)
  
  ## check arguments
  if (nargin < 1)
    help anaers_wavelet;
    error('Less arguments given!');
    return;
  endif
  if (nargin < 2)
    ## fallback values
    par_b = 10;
    par_d = par_b * 2;
    par_e = 0.3;
    par_f = 1.0;
    par_g = 0.7;
  else
    ## fixed wavelet parameters from visual inspection
    par_b = p_ap.wla.par_b;
    par_d = p_ap.wla.par_d;
    par_e = p_ap.wla.par_e;
    par_f = p_ap.wla.par_f;
    par_g = p_ap.wla.par_g;
  endif
  
  ## component 1
  c1ns = par_b + p_c2 + par_d;
  c1yy = wl_exp_negative(c1ns, 5) * par_e; # p1
  
  ## component 2
  c2ns1 = par_b;
  c2ns2 = ceil(p_c2 / 2);
  c2ns3 = p_c2 - c2ns2;
  c2ns4 = par_d;
  c2yy1 = zeros(par_b, 1);
  c2yy2 = wl_exp_negative(c2ns2, 5) * par_f; # p2
  c2yy3 = wl_exp_negative(c2ns3, 5) * par_g; # p3
  c2yy4 = zeros(par_d, 1);
  c2yy = [c2yy1; c2yy2; c2yy3; c2yy4];
  
  ## superimpose components
  r_wl = c1ns;
  r_wm = c1yy + c2yy;
  r_wm = r_wm / max(r_wm);
  
  ## center of distribution
  x = transpose(linspace(1 , r_wl, r_wl));
  wm_mom = sum(r_wm .* x);
  wm_sum = sum(r_wm);
  r_wc = floor(wm_mom / wm_sum);
  
  ## marker points
  r_mx = [1, c1ns, c2ns1 + 1, c2ns1 + c2ns3 + 1, c2ns1 + c2ns2 + c2ns3 + 1];
  r_mv = r_wm(r_mx);
  
endfunction

function [r_yy] = wl_exp_negative(p_ns, p_xm)
  ## Generate wavelet function using f(x) := exp(-x), x in [0,...,5], normalized
  ##
  ## Usage: [r_yy] = wl_exp_negative(p_ns, p_xm)
  ##
  ## p_ns ... number of samples (wavelet length), <uint>
  ## p_xm ... max x coordinate, default = 5
  ## r_yy ... return wavelet function magnitudes [ns x 1]
  
  ## check arguments
  if (nargin < 2)
    p_xm = 5;
  endif
  if (nargin < 1)
    help wl_exp_negative;
    error('Less arguments given!');
    return;
  endif
  
  ## check max x
  xm = max([1, fix(p_xm)]);
  
  ## generate wavelet
  xx = transpose(linspace(0, xm, p_ns));
  yy = exp(-xx);
  r_yy = yy / max(yy);
  
endfunction
