## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## Make a structure containg a structure array of wavelet functions- and parameters (filter bank)
##
## Usage: [r_ws] = anaers_wavelet_mkstruct(p_wt, p_sg, p_l0, p_l1)
##
## p_ap ... analysis parameter structure, <struct>
## r_ws ... return: wavelet structure array, [<struct>]
##   .wm ... array of wavelet function magnitudes [wl x 1], [<double>]
##   .wl ... wavelet length, <uint>
##   .wc ... wavelet center of distribution, sample index, <uint>
##   .mx ... marker point indices, characteristic points [1 x 5], <uint>
##   .mv ... marker point values, characteristic points [1 x 5], <double>
##
## see also: anaers_wavelet
##
function [r_ws] = anaers_wavelet_mkstruct(p_ap)
  
  ## check arguments
  if (nargin < 1)
    help anaers_wavelet_mkstruct;
    error('Less arguments given');
  endif
  
  ## init return value
  r_ws = [];
  
  ## iterate over wavelet array of lengths
  for c2 = [p_ap.wla.ln0 : p_ap.wla.lss : p_ap.wla.ln1]
    ## get wavelet structure by wavelet type
    switch (p_ap.wla.ftp)
      case 'tp2'
        [wm, wl, wc, mx, mv] = anaers_wavelet(p_ap, c2);
      otherwise
        error('anaers_wavelet_mkstruct: wavelet type %s not implemented yet!', p_ap.wla.ftp);
    endswitch
    ## update structure array
    r_ws = [r_ws; hlp_init(wm, wl, wc, mx, mv)];
  endfor
  
endfunction

function [r_ws] = hlp_init(p_wm, p_wl, p_wc, p_mx, p_mv)
  ## Initialize wavelet data structure
  ## p_wm ... array of wavelet function magnitudes [wl x 1], [<double>]
  ## p_wl ... wavelet length, <uint>
  ## p_wc ... wavelet distribution center distance, <uint>
  ## p_mx ... marker point indices, characteristic points [1 x 5], <uint>
  ## p_mv ... marker point values, characteristic points [1 x 5], <double>
  ## r_ws ... return: wavelet data structure, <struct>
  
  ## create structure
  r_ws.wm = transpose(p_wm); # make column vector
  r_ws.wl = p_wl;
  r_ws.wc = p_wc;
  r_ws.mx = transpose(p_mx); # make column vector
  r_ws.mv = transpose(p_mv); # make column vector
  
endfunction
