## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## Compute cross correlation between signal magnitude series and wavelet function
##
## Usage 1: [r_ds] = anaers_wavelet_xcorr2(p_ms, p_i0, p_i1, p_tp, p_sn, p_l0, p_l1, p_sc, p_cn)
## Usage 2: [r_ds] = anaers_wavelet_xcorr2()
##
## Note: When called without parameters, this function returns an empty
##       wavelet analysis data structure.
##
## p_ap ... analysis parameter structure, <struct>
## p_ms ... signal magnitude series [ns x 2], [<float>]
## p_sc ... dataset code, <str>
## p_cn ... channel number, <uint>
## r_ds ... return: cross correlation result structure, see hlp_init(), <struct>
##
## see also: anaers_wavelet_mkstruct
##
function [r_ds] = anaers_wavelet_xcorr2(p_ap, p_ms, p_sc, p_cn)
  
  ## check arguments
  if (nargin == 0)
    r_ds = hlp_init('NN', 0);
    return;
  endif
  if (nargin < 4)
    help anaers_wavelet_xcorr2;
    error('Less arguments given!');
  endif
  
  ## init structure
  r_ds = hlp_init(p_sc, p_cn);
  
  ## update structure with function parameters
  ## signal magnitude max array
  r_ds.ms1.v = p_ms(:, 1);
  ## signal magnitude min array
  r_ds.ms2.v = p_ms(:, 2);
  ## wavelet function type (used to compile wavelet filter bank)
  r_ds.ftp.v = p_ap.wla.ftp;
  ## wavelet shift start index
  r_ds.si0.v = p_ap.wla.si0;
  ## wavelet shift end index
  r_ds.si1.v = p_ap.wla.si1;
  ##  wavelet shift step size
  r_ds.sss.v = p_ap.wla.sss;
  ## number of wavelet shift steps
  r_ds.nss.v = r_ds.si1.v - r_ds.si0.v + 1;
  ## wavelet start length
  r_ds.wl0.v = p_ap.wla.ln0;
  ## wavelet end length
  r_ds.wl1.v = p_ap.wla.ln1;
  ## wavelet length step size
  r_ds.lss.v = p_ap.wla.lss;
  ## number of wavelet length steps
  r_ds.nls.v = r_ds.wl1.v - r_ds.wl0.v + 1;
  
  ## check wavelet length range
  if (r_ds.wl1.v < r_ds.wl0.v)
    error('anaers_wavelet_xcorr2: max wavelet length lower than min wavelet length!');
  endif
  if (r_ds.nls.v < 1)
    error('anaers_wavelet_xcorr2: number of wavelet length steps is zero!');
  endif
  
  ## check shift index range
  if (r_ds.si1.v < r_ds.si0.v)
    error('anaers_wavelet_xcorr2: max shift index lower than min shift index!');
  endif
  if (r_ds.nss.v < 1)
    error('anaers_wavelet_xcorr2: number of wavelet shift steps is zero!');
  endif
  
  ## start time benchmark
  r_ds.btm.v = hlp_benchmark_time();
  
  ## compile wavelet structure array (wavelet filter bank)
  ws = anaers_wavelet_mkstruct(p_ap);
  
  ## max wavelet length
  wlmaxlen = max([ws.wl]);
  
  ## shift index array
  shift_vec = r_ds.si0.v : r_ds.si1.v;
  
  ## extend signal vector if necessary
  ## extension is required if shift end index plus max wavelet length exceeds the signal vector size
  [p_ms, r_ds.errx] = hlp_extend(p_ms, r_ds.si1.v, wlmaxlen);
  
  ## pre-allocate memory for correlation matrix
  r_ds.cmat.v = zeros(r_ds.nls.v, r_ds.nss.v);
  
  ## compute correlation matrix
  for i = 1 : r_ds.lss.v : r_ds.nls.v
    ## iterate over wavelet length vector
    for ii = 1 : r_ds.sss.v : r_ds.nss.v
      ## iterate over index shift vector
      ## shift index of correlation
      ci = shift_vec(ii);
      ## correlation index range, selection of signal samples
      cr = ci : (ci + ws(i).wl - 1);
      ## wavelet magnitudes, multiplier = +1
      cws1 = transpose(ws(i).wm);
      ## wavelet magnitudes, multiplier = -1
      cws2 = -cws1;
      ## compute scalar product (.*)
      ## scale result with index range size (mean)
      ## correlation with positive wavelet, first column of data matrix
      cc1 = mean(p_ms(cr, 1) .* cws1);
      ## correlation with negative wavelet, second column of data matrix
      cc2 = mean(p_ms(cr, 2) .* cws2);
      ## compute correlation value, mean of correlation with positive and negative wavelet function
      r_ds.cmat.v(i, ii) = mean([cc1, cc2]);
    endfor
  endfor
  
  ## find row index, best match (wavelet length)
  [rv, r_ds.bmri.v] = max(max(r_ds.cmat.v, [], 2));
  
  ## corr. matrix row vector, best match
  r_ds.bmcvec.v = r_ds.cmat.v(r_ds.bmri.v, :);
  
  ## wavelet length error state
  if (r_ds.nls.v > 1)
    #r_ds.bmri.v
    switch (r_ds.bmri.v)
    case 1
      ## length is at lower limit of length range
      r_ds.errl.v = 1;
      r_ds.errl.d = 'anaers_wavelet_xcorr2: wavelet length at lower limit! WARNING';
      disp(r_ds.errl.d);
      figure();
      hold on;
      plot(r_ds.ms1.v);
      plot(r_ds.ms2.v);
      hold off;
    case r_ds.nls.v
      ## length is at upper limit of length range
      r_ds.errl.v = 2;
      r_ds.errl.d = 'anaers_wavelet_xcorr2: wavelet length at upper limit! WARNING';
      disp(r_ds.errl.d);
      figure();
      hold on;
      plot(r_ds.ms1.v);
      plot(r_ds.ms2.v);
      hold off;
    otherwise
      ## length is inside length range
      r_ds.errl.v = 0;
      r_ds.errl.d = 'anaers_wavelet_xcorr2: wavelet length inside given range. SUCCESS';
      ## disp(r_ds.errl.d);
    endswitch
  else
    r_ds.errl.v = 0;
    r_ds.errl.d = 'anaers_wavelet_xcorr2: number of length steps lower than 1! ERROR';
    disp(r_ds.errl.d);
  endif
  
  ## find column index of best match (wavelet shift)
  [r_ds.bmcv.v, r_ds.bmci.v] = max(r_ds.bmcvec.v);
  
  ## wavelet shift error state
  if (r_ds.nss.v > 1)
    switch (r_ds.bmci.v)
    case 1
      ## shift index is at lower limit of shift range
      r_ds.errs.v = 1;
      r_ds.errs.d = 'anaers_wavelet_xcorr2: shift index at lower limit! WARNING';
      disp(r_ds.errs.d);
    case r_ds.nss.v
      ## shift index is at upper limit of shift range
      r_ds.errs.v = 2;
      r_ds.errs.d = 'anaers_wavelet_xcorr2: shift index at upper limit! WARNING';
      disp(r_ds.errs.d);
    otherwise
      ## shift index is inside shift range
      r_ds.errs.v = 0;
      r_ds.errs.d = 'anaers_wavelet_xcorr2: wavelet length inside given range. SUCCESS';
      ## disp(r_ds.errs.d);
    endswitch
  else
    r_ds.errs.v = 0;
    r_ds.errs.d = 'anaers_wavelet_xcorr2: number of shift steps lower than 1! ERROR';
    disp(r_ds.errs.d);
  endif
  
  ## stop time benchmark
  r_ds.btm.v = hlp_benchmark_time(r_ds.btm.v);
  
  ## best matching wavelet properties
  bm_ws = ws(r_ds.bmri.v);
  r_ds.bmws.v = shift_vec(r_ds.bmci.v);
  r_ds.bmwm.v = bm_ws.wm;
  r_ds.bmwl.v = bm_ws.wl;
  r_ds.bmwc.v = bm_ws.wc;
  r_ds.bmwx.v = r_ds.bmws.v : (r_ds.bmws.v + r_ds.bmwl.v - 1);
  r_ds.bmplt.v = [r_ds.bmwx.v', r_ds.bmwm.v'];
  r_ds.bmmx1.v = r_ds.bmws.v + bm_ws.mx(1) - 1;
  r_ds.bmmx2.v = r_ds.bmws.v + bm_ws.mx(2) - 1;
  r_ds.bmmx3.v = r_ds.bmws.v + bm_ws.mx(3) - 1;
  r_ds.bmmx4.v = r_ds.bmws.v + bm_ws.mx(4) - 1;
  r_ds.bmmx5.v = r_ds.bmws.v + bm_ws.mx(5) - 1;
  r_ds.bmmv1.v = bm_ws.mv(1);
  r_ds.bmmv2.v = bm_ws.mv(2);
  r_ds.bmmv3.v = bm_ws.mv(3);
  r_ds.bmmv4.v = bm_ws.mv(4);
  r_ds.bmmv5.v = bm_ws.mv(5);
  
  ## print results
  printf('anaers_wavelet_xcorr2: wavelet shift = %d, wavelet length = %d, total length = %d\n', r_ds.bmws.v, r_ds.bmwl.v, r_ds.bmws.v + r_ds.bmwl.v);

endfunction

function [r_vt] = hlp_benchmark_time(p_vt)
  ## Benchmark time tool
  ##
  ## Usage 1: [r_vt] = benchmark_time() ... start computation time benchmark
  ## Usage 2: [r_vt] = benchmark_time(p_vt) ... end computation time benchmark (provide r_vt from benchmark start as argument p_vt)
  ##
  ## p_vt ... time vector [1x3], optional, [<dbl>]
  ## r_vt ... return: time vector, if p_vt is omitted return [now(), 0, 0], otherwise return [t0, now(), delta_t], delta_t in seconds, [<dbl>]
  ##
  ## Example:
  ## start benchmark: initial_vt = benchmark_time()
  ## ... run computations
  ## end benchmark: updated_vt = benchmark_time(initial_vt)
  
  ## check arguments
  if (nargin < 1)
    r_vt = [now(), 0, 0];
    return;
  endif
  
  ## update
  if (max(size(p_vt)) == 3)
    r_vt = p_vt;
    r_vt(2) = now();
    r_vt(3) = (r_vt(2) - r_vt(1)) * 3600 * 24;
  else
    r_vt = p_vt;
  endif
  
endfunction

function [r_ds] = hlp_meta(p_sc, p_cn)
  ## Create dataset metadata structure
  ## p_sc  ... dataset code, <str>
  ## p_cn  ... channel number, <uint>
  ## r_ds  ... return: dataset group definition data structure, <struct>
  
  r_ds.obj = [];
  r_ds.ver = [];
  r_ds.algoname = [];
  r_ds.algoversion = [];
  r_ds.chnum = struct_objattrib('chnum', sprintf('%d', p_cn), 'channel number');
  if (p_cn == 1)
    r_ds.chncode = struct_objattrib('chncode', 'C', 'channel code');
    r_ds.chnamelong = struct_objattrib('chnamelong', 'Compression wave', 'channel name, long');
    r_ds.chnameshort = struct_objattrib('chnameshort', 'C-wave', 'channel name, short');
  else
    r_ds.chncode = struct_objattrib('chncode', 'S', 'channel code');
    r_ds.chnamelong = struct_objattrib('chnamelong', 'Shear wave', 'channel name, long');
    r_ds.chnameshort = struct_objattrib('chnameshort', 'S-wave', 'channel name, short');
  endif
  r_ds.dscode = struct_objattrib('dscode', p_sc, 'dataset code');
  r_ds.dscodetex = struct_objattrib('dscodetex', strrep(p_sc, '_', '\_'), 'dataset code, TeX escaped');

endfunction

function [r_ds] = hlp_init(p_sc, p_cn)
  ## Initialize wavelet analysis result data structure
  ## p_sc ... dataset code, <str>
  ## p_cn ... channel number, <uint>
  ## r_ds ... return: 
  
  ## init data structure with metadata
  r_ds = hlp_meta(p_sc, p_cn);
  
  ## create wavelet analysis data structure
  r_ds.obj = 'struct_ana_wlaxc2';
  r_ds.ver = int16([1, 0]);
  r_ds.algoname = 'MIN/MAX cross-correlation wavelet analysis';
  r_ds.algoversion = int16([1, 0]);
  r_ds.btm = struct_objdata('bmarktime', 'double_arr', [], 'sec', 'benchmark time');
  ## signal data, min/max curves
  r_ds.ms1 = struct_objdata('magmax', 'double_arr', [], 'V', 'signal magnitude max curve, column vector');
  r_ds.ms2 = struct_objdata('magmin', 'double_arr', [], 'V', 'signal magnitude min curve, column vector');
  ## wavelet analysis parameters
  r_ds.ftp = struct_objattrib('functype', '', 'wavelet function type');
  r_ds.si0 = struct_objdata('shiftidx0', 'uint', 0, '#', 'wavelet shift start index, sample index space');
  r_ds.si1 = struct_objdata('shiftidx1', 'uint', 0, '#', 'wavelet shift end index, sample index space');
  r_ds.sss = struct_objdata('shiftstepsize', 'uint', 0, '#', 'wavelet shift step size');
  r_ds.nss = struct_objdata('numshiftsteps', 'uint', 0, '#', 'number of wavelet shift steps');
  r_ds.wl0 = struct_objdata('length0', 'uint', 0, '#', 'wavelet start length, number of samples');
  r_ds.wl1 = struct_objdata('length1', 'uint', 0, '#', 'wavelet end length, number of samples');
  r_ds.lss = struct_objdata('lengthstepsize', 'uint', 0, '#', 'wavelet length step size');
  r_ds.nls = struct_objdata('numlengthsteps', 'uint', 0, '#', 'number wavelet length steps');
  ## cross correlation, find best match
  r_ds.cmat = struct_objdata('corrmat', 'double_mat', [], 'V', 'wavelet correlation matrix, rows = length steps, columns = shift steps');
  r_ds.bmcvec = struct_objdata('bmcorrvec', 'double_arr', [], 'V', 'best match, correlation vector, row vector of correlation matrix');
  r_ds.bmri = struct_objdata('bmrowidx', 'uint', 0, '#', 'best match, correlation matrix row index');
  r_ds.bmci = struct_objdata('bmcolidx', 'uint', 0, '#', 'best match, correlation matrix column index');
  r_ds.bmcv = struct_objdata('bmcorrval', 'double', 0, '/', 'best match, correlation value');
  ## best matching wavelet properties
  r_ds.bmwm = struct_objdata('bmwmag', 'double_arr', [], 'V', 'best match, wavelet magnitude array, y axis');
  r_ds.bmwx = struct_objdata('bmwidx', 'uint_arr', [], '#', 'best match, wavelet index array, sample index space, x axis');
  r_ds.bmws = struct_objdata('bmwshift', 'uint', 0, '#', 'best match, wavelet shift, sample index space');
  r_ds.bmwl = struct_objdata('bmwlength', 'uint', 0, '#', 'best match, wavelet length, number of samples');
  r_ds.bmwc = struct_objdata('bmwcenter', 'uint', 0, '#', 'best match, wavelet center index, sample index space');
  r_ds.bmwe = struct_objdata('bmwend', 'uint', 0, '#', 'best match, wavelet end index, sample index space');
  r_ds.bmplt = struct_objdata('bmplt', 'double_mat', [], '', 'best match, plot data');
  ## marker points, characteristic points
  r_ds.bmmx1 = struct_objdata('bmmx1', 'uint', [], '', 'best match, marker point 1 sample index');
  r_ds.bmmx2 = struct_objdata('bmmx2', 'uint', [], '', 'best match, marker point 2 sample index');
  r_ds.bmmx3 = struct_objdata('bmmx3', 'uint', [], '', 'best match, marker point 3 sample index');
  r_ds.bmmx4 = struct_objdata('bmmx4', 'uint', [], '', 'best match, marker point 4 sample index');
  r_ds.bmmx5 = struct_objdata('bmmx5', 'uint', [], '', 'best match, marker point 5 sample index');
  r_ds.bmmv1 = struct_objdata('bmmv1', 'double', [], '', 'best match, marker point 1 magnitude value');
  r_ds.bmmv2 = struct_objdata('bmmv2', 'double', [], '', 'best match, marker point 2 magnitude value');
  r_ds.bmmv3 = struct_objdata('bmmv3', 'double', [], '', 'best match, marker point 3 magnitude value');
  r_ds.bmmv4 = struct_objdata('bmmv4', 'double', [], '', 'best match, marker point 4 magnitude value');
  r_ds.bmmv5 = struct_objdata('bmmv5', 'double', [], '', 'best match, marker point 5 magnitude value');
  ## error states
  r_ds.errx = struct_objdata('errextend', 'bool', false, '/', 'error state, required extension of signal magnitude vector due to max wavelet length');
  r_ds.errl = struct_objdata('errlength', 'int', -1, '/', 'error state, wavelet length variation limit error');
  r_ds.errs = struct_objdata('errshift', 'int', -1, '/', 'error state, wavelet shift variation limit error');
  
endfunction

function [r_ms, r_ex] = hlp_extend(p_ms, p_ie, p_le)
  ## Extend signal array
  ## Note: The signal magnitude vector need to be able to run the cross-correlation
  ## from start to end index regardless of the wavelet length.
  ## This is to avoid that the cross-correlation exceeds the signal magnitude array size.
  ## If necessary the signal magnitude magnitude array is extended with a sufficient number of zeros (padding).
  ##
  ## p_ms ... signal magnitude series [n x 1], [<float>]
  ## p_ie ... wavelet shift, end index, <int>
  ## p_le ... wavelet end length (max. length), <ADE/struct>
  
  ## init error state
  r_ex = struct_objdata('errextend', 'bool', false, '', 'error state');
  
  ## extend signal vector, if necessary
  idx_max = p_ie + p_le;
  if (size(p_ms, 1) < idx_max)
    ## extend signal vectors
    r_ms = [p_ms; zeros(p_le + 1, 2)];
    r_ex.v = true;
    r_ex.d = 'anaers_wavelet_xcorr2: signal vectors extended!';
    disp(r_ex.d);
  else
    ## no extension required, vectors are long enough
    r_ms = p_ms;
    r_ex.v = false;
    r_ex.d = 'anaers_wavelet_xcorr2: no need to extend signal vectors';
  endif
  
endfunction
