## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## Create wavelet design schema graphic
##
## Usage: anaers_wldesign()
##
## see also: 
##
function anaers_wldesign()
  
  ## read analysis parameters
  ap = anaers_param();
  
  ## part 1
  xshift = 4 * pi;
  nosz1 = 22;
  scal = 0.3;
  x1 = linspace(0, 2 * nosz1 * pi, 1000);
  x1s = x1 + xshift;
  e10p = exp(-x1/30);
  e10m = -e10p;
  e11p = e10p * scal;
  e11m = e10m * scal;
  y10 = sin(x1);
  y11 = y10 .* e10p * scal;
  
  ## part 2
  xshift = 8 * pi;
  nosz2 = 8;
  scal = 1;
  x2 = linspace(0, 2 * nosz2 * pi, 400);
  x2s = x2 + xshift;
  e20p = exp(-x2/14);
  e20m = -e20p;
  e21p = e20p * scal;
  e21m = e20m * scal;
  y20 = sin(x2);
  y21 = y20 .* e20p * scal;
  
  ## part 3
  xshift = 24 * pi;
  nosz3 = 8;
  scal = 0.7;
  x3 = linspace(0, 2 * nosz3 * pi, 400);
  x3s = x3 + xshift;
  e30p = exp(-x3/14);
  e30m = -e30p;
  e31p = e30p * scal;
  e31m = e30m * scal;
  y30 = sin(x2);
  y31 = y30 .* e30p * scal;
  
  ## plot wavelet design schema
  figure();
  hold on;
  plot(x1s, y11, '-r'); # oscillation, part 1
  plot(x1s, e11p, '-r'); # max envelope, part 1
  plot(x1s, e11m), '-r'; # min envelope, part 1
  plot(x2s, y21, '-b'); # oscillation, part 2
  plot(x2s, e21p, '-b'); # max envelope, part 2
  plot(x2s, e21m, '-b'); # min envelope, part 2
  plot(x3s, y31, '-g'); # oscillation, part 3
  plot(x3s, e31p, '-g'); # max envelope, part 3
  plot(x3s, e31m, '-g'); # min envelope, part 3
  title('Wavelet design schema');
  xlabel('Sample index i [ ]');
  ylabel('Magnitude [ ]');
  hold off;
  
  ## prepare data for TeX export
  m1 = [x1s', y11', e11p', e11m'];
  m2 = [x2s', y21', e21p', e21m'];
  m3 = [x3s', y31', e31p', e31m'];
  
  ## create export data structure
  ## data, part 1 to 3
  ds.p1 = struct_objdata('p1', 'double_mat', m1, '', 'disturbance part 1');
  ds.p2 = struct_objdata('p2', 'double_mat', m2, '', 'disturbance part 2');
  ds.p3 = struct_objdata('p3', 'double_mat', m3, '', 'disturbance part 3');
  ## interval limits
  ds.p0x0 = struct_objdata('p0x0', 'double', 0, '', 'disturbance offset, x axis start value');
  ds.p0x1 = struct_objdata('p0x1', 'double', x1s(1), '', 'disturbance offset, x axis end value');
  ds.p1x0 = struct_objdata('p1x0', 'double', x1s(1), '', 'disturbance part 1, x axis start value');
  ds.p1x1 = struct_objdata('p1x1', 'double', x1s(end), '', 'disturbance part 1, x axis end value');
  ds.p2x0 = struct_objdata('p2x0', 'double', x2s(1), '', 'disturbance part 2, x axis start value');
  ds.p2x1 = struct_objdata('p2x1', 'double', x2s(end), '', 'disturbance part 2, x axis end value');
  ds.p3x0 = struct_objdata('p3x0', 'double', x3s(1), '', 'disturbance part 3, x axis start value');
  ds.p3x1 = struct_objdata('p3x1', 'double', x3s(end), '', 'disturbance part 3, x axis end value');
  ## add annotation
  ds.xlbl1 = struct_objattrib('xlbl1', 'Sample index', 'x axis label');
  ds.xunit1 = struct_objattrib('xunit1', '', 'x axis unit');
  ds.xlbl2 = struct_objattrib('xlbl2', 'Time', 'x axis label');
  ds.xunit2 = struct_objattrib('xunit2', 'sec', 'x axis unit');
  ds.ylbl = struct_objattrib('ylbl', 'Magnitude', 'y axis label');
  ds.yunit = struct_objattrib('yunit', 'V', 'y axis unit');
  ds.title = struct_objattrib('title', 'EM Disturbance', 'figure title');
  
  ## TeX export
  if (ap.post.save_tex)
    ## output file path
    ofp = fullfile(ap.scpath, 'export', 'schema', 'wldesign');
    ## export data structure to TeX file
    errcode = tex_struct_export(ds, 'wldesign', ofp);
  endif
  
endfunction
