## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## TeXify atomic data element
##
## Usage: [r_tc] = struct_objdata_totex(p_ds, p_dn)
##
## p_ds ... data structure "objdata", <ADE/struct>
## p_dn ... data structure name (used as variable name prefix), <str>
## r_tc ... return: TeX code representation of structure element(s), {<str>}
##
## see also: struct_objattrib
##
function [r_tc] = struct_objdata_totex(p_ds, p_dn)
  
  ## check arguments
  if (nargin < 2)
    help struct_objattrib_totex;
    error('Less arguments given!');
  endif
  
  ## check type of data structure
  if (not(isfield(p_ds, 'obj')))
    warning('Data structure is not of type DATA! name = %s', p_dn);
    r_tc = '';
    return;
  else
    if (not(p_ds.obj == 'ADE'))
      warning('Data structure is not of type DATA! name = %s', p_dn);
      r_tc = '';
      return;
    endif
  endif
  
  ## render data structure to TeX code
  r_tc = {};
  ##r_tc = [r_tc; sprintf('\\def\\%sobj{%s}', p_dn, p_ds.obj)];
  ##r_tc = [r_tc; sprintf('\\def\\%sver{%d.%d}', p_dn, p_ds.ver(1), p_ds.ver(2))];
  ##r_tc = [r_tc; sprintf('\\def\\%st{%s}', p_dn, p_ds.t)];
  ##r_tc = [r_tc; sprintf('\\def\\%svt{%s}', p_dn, p_ds.vt)];
  r_tc = [r_tc; sprintf('\\def\\%su{%s}', p_dn, p_ds.u)];
  ##r_tc = [r_tc; sprintf('\\def\\%sd{%s}', p_dn, p_ds.d)];
  r_tc = hlp_value(r_tc, p_dn, p_ds.vt, p_ds.v);
  
endfunction

############################################################################################################################################
## Helper function: append TeX code representation of value to TeX code listing
## p_vt ... value type id or string, <int> or <str>
## r_md ... return: meta data, <struct>
## r_dv ... return: default value of given value type, <any_GNU_octve_value>
function [r_tc] = hlp_value(p_tc, p_dn, p_vt, p_v)
  
  r_tc = p_tc;
  
  ## switch value type
  switch (p_vt)
    case 'str'
      ## string
      r_tc = [r_tc; sprintf('\\def\\%sv{%s}', p_dn, p_v)];
    case 'str_arr'
      ## string array (cell array 1D)
      for i = 1 : max(size(p_v))
        r_tc = [r_tc; sprintf('\\def\\%svr%d{%s}', p_dn, p_v{i}, i)];
      endfor
    case 'str_mat'
      ## string array (cell array 2D)
      for i = 1 : size(p_v, 1)
        for ii = 1 : size(p_v, 2)
          r_tc = [r_tc; sprintf('\\def\\%svr%dc%d{%s}', p_dn, p_v{i, ii}, i, ii)];
        endfor
      endfor
    case 'bool'
      ## boolean
      r_tc = hlp_scal(r_tc, p_dn, p_v, '%d');
    case 'bool_arr'
      ## boolean array
      r_tc = hlp_arr(r_tc, p_dn, p_v, '%d');
    case 'bool_mat'
      ## boolean matrix
      r_tc = hlp_mat(r_tc, p_dn, p_v, '%d');
    case 'uint'
      ## unsigned integer
      r_tc = hlp_scal(r_tc, p_dn, p_v, '%d');
    case 'uint_arr'
      ## unsigned integer array
      r_tc = hlp_arr(r_tc, p_dn, p_v, '%d');
    case 'uint_mat'
      ## unsigned integer matrix
      r_tc = hlp_mat(r_tc, p_dn, p_v, '%d');
    case 'int'
      ## integer
      r_tc = hlp_scal(r_tc, p_dn, p_v, '%d');
    case 'int_arr'
      ## integer array
      r_tc = hlp_arr(r_tc, p_dn, p_v, '%d');
    case 'int_mat'
      ## integer matrix
      r_tc = hlp_mat(r_tc, p_dn, p_v, '%d');
    case 'float'
      ## single precision
      r_tc = hlp_scal(r_tc, p_dn, p_v, '%.8f');
    case 'float_arr'
      ## single precision array
      r_tc = hlp_arr(r_tc, p_dn, p_v, '%.8f');
    case 'float_mat'
      ## single precision matrix
      r_tc = hlp_mat(r_tc, p_dn, p_v, '%.8f');
    case 'dbl'
      ## double precision
      r_tc = hlp_scal(r_tc, p_dn, p_v, '%.8f');
    case 'dbl_arr'
      ## double precision array
      r_tc = hlp_arr(r_tc, p_dn, p_v, '%.8f');
    case 'dbl_mat'
      ## double precision matrix
      r_tc = hlp_mat(r_tc, p_dn, p_v, '%.8f');
    otherwise
      warning('Unknown value type name: %s', p_vt);
  endswitch
  
endfunction

############################################################################################################################################
## Helper function: render scalar to TeX code
function [r_tc] = hlp_scal(p_tc, p_dn, p_v, p_f)
  
  r_tc = p_tc;
  
  if isempty(p_v)
    return;
  endif
  
  r_tc = [r_tc; sprintf(['\\def\\%sv{', p_f, '}'], p_dn, p_v)];
  
endfunction

############################################################################################################################################
## Helper function: render array to TeX code
function [r_tc] = hlp_arr(p_tc, p_dn, p_v, p_f)
  
  p_v = p_v(:);
  
  r_tc = p_tc;
  
  if isempty(p_v)
    return;
  endif
  
  r_tc = [r_tc; sprintf('\\begin{filecontents}{%sv.data}', p_dn)];
  r_tc = [r_tc; hlp_colhead(size(p_v, 2))];
  for i = 1 : size(p_v, 1)
    r_tc = [r_tc; hlp_row(i, p_f, p_v)];
  endfor
  r_tc = [r_tc; sprintf('\\end{filecontents}')];
  
  r_tc = [r_tc; sprintf('\\begin{filecontents}{%sv.stats}', p_dn)];
  r_tc = [r_tc; hlp_colhead(size(p_v, 2))];
  r_tc = [r_tc; hlp_row(1, p_f, max(p_v))];
  r_tc = [r_tc; hlp_row(1, p_f, min(p_v))];
  r_tc = [r_tc; sprintf('\\end{filecontents}')];
  
endfunction

############################################################################################################################################
## Helper function: render matrix to TeX code
function [r_tc] = hlp_mat(p_tc, p_dn, p_v, p_f)
  
  r_tc = p_tc;
  
  if isempty(p_v)
    return;
  endif
  
  r_tc = [r_tc; sprintf('\\begin{filecontents}{%sv.data}', p_dn)];
  r_tc = [r_tc; hlp_colhead(size(p_v, 2))];
  for i = 1 : size(p_v, 1)
    r_tc = [r_tc; hlp_row(i, p_f, p_v)];
  endfor
  r_tc = [r_tc; sprintf('\\end{filecontents}')];
  
  r_tc = [r_tc; sprintf('\\begin{filecontents}{%sv.stats}', p_dn)];
  r_tc = [r_tc; hlp_colhead(size(p_v, 2))];
  r_tc = [r_tc; hlp_row(1, p_f, max(p_v))];
  r_tc = [r_tc; hlp_row(1, p_f, min(p_v))];
  r_tc = [r_tc; sprintf('\\end{filecontents}')];
  
endfunction

############################################################################################################################################
## Helper function: render table column head
function [r_str] = hlp_colhead(p_num)
  
  colarr = 1 : p_num;
  str1 = '    idx';
  str2 = sprintf('   c%d', colarr);
  
  r_str = [str1, str2];
  
endfunction

############################################################################################################################################
## Helper function: render table row
function [r_str] = hlp_row(p_i, p_f, p_v)
  
  fmt = ['   ', p_f];
  str1 = sprintf('    %d', p_i);
  str2 = sprintf(fmt, p_v(p_i, :));
  
  r_str = [str1, str2];
  
endfunction
