## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## Define 1D-array variable as TeX table
##
## Usage 1: [r_tc] = tex_def_tabarray(p_sp, p_fmt, p_val, p_tc) ... update existing TeX code listing
## Usage 2: [r_tc] = tex_def_tabarray(p_sp, p_fmt, p_val) ... return TeX code for variable only
##
## p_vn  ... variable name, <str>
## p_fmt ... variable format (see sprintf), <str>
## p_val ... variable value, [<number>] or {<string>}
## p_tc  ... TeX code listing to be updated, optional, {<str>}
## r_tc  ... return: TeX code listing (usage 1) or TeX code for variable (usage 2), {<str>} or <str>
##
## see also: tex_settings
##
## Output format:
## \begin{filecontents}{<variable-name>}
##    idx ; c1
##    1   ; x_1
##    ... ; ...
##    i   ; x_i
##    ... ; ...
##    n   ; x_n
## \end{filecontents}
##
## x_i can be a number or a string (strings must not contain semicola!)
## i = 1,n ... row index
##
function [r_tc] = tex_def_tabarray(p_vn, p_fmt, p_vv, p_tc)
  
  ## check arguments
  if (nargin < 3)
    help tex_def_tabarray;
    error('Less arguments given!');
  endif
  if (nargin < 4)
    append_list = false;
  else
    append_list = true;
  endif
  
  ## load TeX settings data structure
  dstex = tex_settings();
  
  ## render TeX code
  if isempty(p_vv)
    if append_list
      r_tc = p_tc;
    else
      r_tc = '';
    endif
  else
    ## created TeX environment header
    str_head = [dstex.begin_fc, '{', p_vn, '}'];
    ## create TeX environment footer
    str_foot = dstex.end_fc;
    ## add column header, row 1, semicolon separated
    str_rows = {'idx ; c1'};
    if iscell(p_vv)
      ## handle cell array
      for i = 1 : max(size(p_vv))
        ## add row for each array item
        str_rows = [str_rows; sprintf(['%d ; ', p_fmt], i, p_vv{i})];
      endfor
    else
      ## handle number array
      for i = 1 : max(size(p_vv))
        ## add row for each array item
        str_rows = [str_rows; sprintf(['%d ; ', p_fmt], i, p_vv(i))];
      endfor
    endif
    if append_list
      ## append to given list: environment header, rows, environment footer
      r_tc = [p_tc; str_head; str_rows; str_foot];
    else
      ## return list: environment header, rows, environment footer
      r_tc = [str_head; str_rows; str_foot];
    endif
  endif
  
endfunction
