## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## Define comma-separated array variable in TeX format
##
## Usage 1: [r_tc] = tex_def_tabmatrix(p_sp, p_fmt, p_val, p_tc) ... update existing TeX code listing
## Usage 2: [r_tc] = tex_def_tabmatrix(p_sp, p_fmt, p_val) ... return TeX code for variable only
##
## p_vn  ... variable name, <str>
## p_fmt ... variable format (see sprintf), <str>
## p_val ... variable value, [<number>] or {<string>}
## p_tc  ... TeX code listing to be updated, optional, {<str>}
## r_tc  ... return: TeX code listing (usage 1) or TeX code for variable (usage 2), {<str>} or <str>
##
## see also: tex_settings
##
## Output format:
## \begin{filecontents}{<variable-name>}
##    idx ; c1    ; ... ; cj    ; ... ; cm
##    1   ; x_1,1 ; ... ; x_1,j ; ... ; x_1,m
##    ... ; ...   ; ... ; ...   ; ... ; ...
##    i   ; x_i,1 ; ... ; x_i,j ; ... ; x_i,m
##    ... ; ...   ; ... ; ...   ; ... ; ...
##    n   ; x_n,1 ; ... ; x_n,j ; ... ; x_n,m
## \end{filecontents}
##
## x_i,j can be a number or a string (strings must not contain semicola!)
## i = 1,n ... row index
## j = 1,m ... column index
##
function [r_tc] = tex_def_tabmatrix(p_vn, p_fmt, p_vv, p_tc)
  
  ## check arguments
  if (nargin < 3)
    help tex_def_tabmatrix;
    error('Less arguments given!');
  endif
  if (nargin < 4)
    append_list = false;
  else
    append_list = true;
  endif
  
  ## load TeX settings data structure
  dstex = tex_settings();
  
  ## render TeX code
  if isempty(p_vv)
    if append_list
      r_tc = p_tc;
    else
      r_tc = '';
    endif
  else
    ## number of rows
    nrow = size(p_vv, 1);
    ## number of columns
    ncol = size(p_vv, 2);
    ## created TeX environment header
    str_head = [dstex.begin_fc, '{', p_vn, '}'];
    ## create TeX environment footer
    str_foot = dstex.end_fc;
    ## add column header, row 1, semicolon separated
    str_rows = {hlp_colhead(ncol)};
    if iscell(p_vv)
      ## handle cell array
      for i = 1 : nrow
        ## add row for each row of array
        str_rows = [str_rows; hlp_row_cellstr(i, p_fmt, {p_vv{i, :}})];
      endfor
    else
      ## handle number array
      for i = 1 : nrow
        ## add row for each row of array
        str_rows = [str_rows; hlp_row_numeric(i, p_fmt, p_vv(i, :))];
      endfor
    endif
    if append_list
      ## append to given list: environment header, rows, environment footer
      r_tc = [p_tc; str_head; str_rows; str_foot];
    else
      ## return list: environment header, rows, environment footer
      r_tc = [str_head; str_rows; str_foot];
    endif
  endif
  
endfunction

############################################################################################################################################
function [r_str] = hlp_colhead(p_ncol)
  ## Helper function: render table column head
  
  ## create index array
  colarr = 1 : p_ncol;
  
  ## column 1, index
  str1 = 'idx';
  
  ## add column header for all array columns
  str2 = sprintf(' ; c%d', colarr);
  
  ## return table header
  r_str = [str1, str2];
  
endfunction

############################################################################################################################################
function [r_str] = hlp_row_numeric(p_i, p_f, p_v)
  ## Helper function: render table row, numeric
  
  ## define format string (sprintf)
  fmt = [' ; ', p_f];
  
  ## add current row index
  str1 = sprintf('%d', p_i);
  
  ## add values of current row of array, column 1 to m
  str2 = sprintf(fmt, p_v);
  
  # return current row
  r_str = [str1, str2];
  
endfunction

############################################################################################################################################
function [r_str] = hlp_row_cellstr(p_i, p_f, p_v)
  ## Helper function: render table row, cell string
  
  ## define format string (sprintf)
  fmt = [' ; ', p_f];
  
  ## add current row index
  str1 = sprintf('%d', p_i);
  
  ## add values of current row of array, column 1 to m
  str2 = sprintf(fmt, p_v{:});
  
  # return current row
  r_str = [str1, str2];
  
endfunction
