## Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
## License: MIT
## This file is part of the PhD thesis of Jakob Harden.
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
## documentation files (the “Software”), to deal in the Software without restriction, including without 
## limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
## the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
## 
## THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
## THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
##
## TeXify atomic data element
##
## Usage: [r_tc] = tex_struct_objdata(p_ds, p_sp, p_px)
##
## p_ds ... atomic data structure "objdata", <ADE/struct>
## p_sp ... structure path prefix, <str>
## p_px ... structure path prefix, optional, <str>
## r_tc ... return: TeX code representation of structure element(s), {<str>}
##
## see also: struct_objdata
##
function [r_tc] = tex_struct_objdata(p_ds, p_sp, p_px)
  
  ## check arguments
  if (nargin < 2)
    help tex_struct_objdata;
    error('Less arguments given!');
  endif
  if (nargin < 3)
    spath = p_sp;
  else
    spath = [p_px, p_sp];
  endif
  
  ## check type of data structure
  if (not(isfield(p_ds, 'obj')))
    warning('Data structure is not of type DATA! name = %s', p_sp);
    r_tc = '';
    return;
  else
    if (not(p_ds.obj == 'ADE'))
      warning('Data structure is not of type DATA! name = %s', p_sp);
      r_tc = '';
      return;
    endif
  endif
  
  ## render data structure to TeX code
  r_tc = {};
  r_tc = tex_def_scalar([spath, '.obj'], '%s', p_ds.obj, r_tc);
  r_tc = tex_def_csvarray([spath, '.ver'], '%d', p_ds.ver, r_tc);
  r_tc = tex_def_scalar([spath, '.t'], '%s', p_ds.t, r_tc);
  r_tc = tex_def_scalar([spath, '.d'], '%s', p_ds.d, r_tc);
  switch (p_ds.u)
    case {'/', '#'}
      ## remove unit for scalar numbers
      r_tc = tex_def_scalar([spath, '.u'], '%s', '', r_tc);
    otherwise
      r_tc = tex_def_scalar([spath, '.u'], '%s', p_ds.u, r_tc);
  endswitch
  r_tc = tex_def_scalar([spath, '.vt'], '%s', p_ds.vt, r_tc);
  r_tc = sub_value(r_tc, [spath, '.v'], p_ds.vt, p_ds.v);
  
endfunction

############################################################################################################################################
function [r_tc] = sub_value(p_tc, p_sp, p_vt, p_vv)
  ## Append TeX code representation of value to TeX code listing
  ## p_tc ... TeX code listing, {<str>}
  ## p_sp ... structure path, <str>
  ## p_vt ... value type, <str>
  ## p_vv ... variable value, <any>
  ## r_tc ... return: meta data, <struct>
  
  ## init return value
  r_tc = p_tc;
  
  ## switch value type
  switch (p_vt)
    case 'str'
      ## string
      r_tc = tex_def_scalar(p_sp, '%s', p_vv, r_tc);
    case 'str_arr'
      ## string array (cell array 1D)
      r_tc = tex_def_tabarray(p_sp, '%s', p_vv, r_tc);
    case 'str_mat'
      ## string array (cell array 2D)
      r_tc = tex_def_tabmatrix(p_sp, '%s', p_vv, r_tc);
    case 'bool'
      ## boolean
      r_tc = tex_def_scalar(p_sp, '%d', p_vv, r_tc);
    case 'bool_arr'
      ## boolean array
      r_tc = tex_def_tabarray(p_sp, '%d', p_vv, r_tc);
    case 'bool_mat'
      ## boolean matrix
      r_tc = tex_def_tabmatrix(p_sp, '%d', p_vv, r_tc);
    case 'uint'
      ## unsigned integer
      r_tc = tex_def_scalar(p_sp, '%d', p_vv, r_tc);
    case 'uint_arr'
      ## unsigned integer array
      r_tc = tex_def_tabarray(p_sp, '%d', p_vv, r_tc);
    case 'uint_mat'
      ## unsigned integer matrix
      r_tc = tex_def_tabmatrix(p_sp, '%d', p_vv, r_tc);
    case 'int'
      ## integer
      r_tc = tex_def_scalar(p_sp, '%d', p_vv, r_tc);
    case 'int_arr'
      ## integer array
      r_tc = tex_def_tabarray(p_sp, '%d', p_vv, r_tc);
    case 'int_mat'
      ## integer matrix
      r_tc = tex_def_tabmatrix(p_sp, '%d', p_vv, r_tc);
    case 'float'
      ## single precision
      r_tc = tex_def_scalar(p_sp, '%.10f', p_vv, r_tc);
    case 'float_arr'
      ## single precision array
      r_tc = tex_def_tabarray(p_sp, '%.10f', p_vv, r_tc);
    case 'float_mat'
      ## single precision matrix
      r_tc = tex_def_tabmatrix(p_sp, '%.10f', p_vv, r_tc);
    case 'dbl'
      ## double precision
      r_tc = tex_def_scalar(p_sp, '%.14f', p_vv, r_tc);
    case 'dbl_arr'
      ## double precision array
      r_tc = tex_def_tabarray(p_sp, '%.14f', p_vv, r_tc);
    case 'dbl_mat'
      ## double precision matrix
      r_tc = tex_def_tabmatrix(p_sp, '%.14f', p_vv, r_tc);
    otherwise
      warning('Unknown value type name: %s', p_vt);
  endswitch
  
endfunction
